@@ -1,7 +1,7 @@
 /* Main command processing loop.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -33,6 +33,11 @@ is what most commands require. */
 
 #define NORMALIZE(x)  { x = (x)<0 ? 1 : (x); }
 
+/* The length of the static message buffer. It must be larger by a factor
+	of about three of the maximum screen width as UTF-8 encoded characters
+	might take several characters per screen position. */
+
+#define MAX_MESSAGE_SIZE (1024)
 
 /* Here, given a mask represent a user flag and an integer i, we do as follows:
 	i < 0 : toggle flag;
@@ -44,6 +49,10 @@ is what most commands require. */
 	if ((i)<0) (b)->x = !(b)->x;\
 	else (b)->x = ((i) != 0);\
 }
+#define SET_GLOBAL_FLAG(c,f) {\
+	if ((c)<0) (f) = !(f);\
+	else (f) = ((c) != 0);\
+}
 
 
 /* Converts a non-positive result from request_number() to OK if the
@@ -64,16 +73,16 @@ function was aborted or not-a-number error if an invalid number was read. */
 	zero means off, no argument means toggle.
 
 	If there is a string argument (i.e. p != NULL), it is assumed that the
-	action will consume *p -- it ends up being free()d or stored
+	action will consume p -- it ends up being free()d or stored
 	somewhere. Though efficient, this has lead to some memory leaks (can you
 	find them?). */
 
 
 int do_action(buffer *b, action a, int c, unsigned char *p) {
-	
+	static char msg[MAX_MESSAGE_SIZE];
 	line_desc *next_ld;
 	HIGHLIGHT_STATE next_line_state;
-	int i, error = OK, col;
+	int i, error = OK, recording, col;
 	char *q;
 
 	assert_buffer(b);
@@ -82,7 +91,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 	stop = FALSE;
 
-	if (b->recording) record_action(b->cur_macro, a, c, p, b->opt.verbose_macros);
+	if (b->recording) record_action(b->cur_macro, a, c, p, verbose_macros);
 
 	switch(a) {
 		
@@ -109,7 +118,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return stop ? STOPPED : error ;
 
 	case QUIT_A:
-		if (modified_buffers() && !request_response(b, "Some documents have not been saved; are you sure?", FALSE)) return ERROR;
+		if (modified_buffers() && !request_response(b, info_msg[SOME_DOCUMENTS_ARE_NOT_SAVED], FALSE)) return ERROR;
 		close_history();
 		unset_interactive_mode();
 		exit(0);
@@ -207,6 +216,48 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		for(i = 0; i < c && !(error = search_word(b, -1)) && !stop; i++);
 		return stop ? STOPPED : error;
 
+   case DELETEPREVWORD_A:
+		recording = b->recording;
+		b->recording = 0;
+		NORMALIZE(c);
+		delay_update();
+		start_undo_chain(b);
+		for(i = 0; i < c && !error && !stop; i++) {
+				int right_line = b->cur_line;
+				int right_pos  = b->cur_pos;
+				if(!(error = do_action(b, PREVWORD_A, 1, NULL))) {
+					int left_line = b->cur_line;
+					int left_pos  = b->cur_pos;
+						goto_line(b, right_line);
+					goto_pos(b, right_pos);
+					while(!error && !stop && (b->cur_line > left_line || b->cur_pos > left_pos)) {
+						error = do_action(b, BACKSPACE_A, 1, NULL);
+					}
+				}
+			}
+		end_undo_chain(b);
+		b->recording = recording;
+		return stop ? STOPPED : error;
+
+   case DELETENEXTWORD_A:
+		recording = b->recording;
+		b->recording = 0;
+		NORMALIZE(c);
+		delay_update();
+		start_undo_chain(b);
+		for(i = 0; i < c && !error && !stop; i++) {
+				int left_line = b->cur_line;
+				int left_pos  = b->cur_pos;
+				if(!(error = do_action(b, NEXTWORD_A, 1, NULL))) {
+					while(!error && !stop && (b->cur_line > left_line || b->cur_pos > left_pos)) {
+						error = do_action(b, BACKSPACE_A, 1, NULL);
+					}
+				}
+			}
+		end_undo_chain(b);
+		b->recording = recording;
+		return stop ? STOPPED : error;
+
 	case MOVEEOW_A:
 		move_to_eow(b);
 		return OK;
@@ -243,11 +294,15 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			b->bookmark[c].pos = b->cur_pos;
 			b->bookmark[c].line = b->cur_line;
 			b->bookmark_mask |= (1 << c);
+			snprintf(msg, MAX_MESSAGE_SIZE, "Bookmark %d set", c-1);
+			print_message(msg);
 			break;
 		case UNSETBOOKMARK_A:
 			if (! (b->bookmark_mask & (1 << c)))
 				return BOOKMARK_NOT_SET;
 			b->bookmark_mask &= ~(1 << c);
+			snprintf(msg, MAX_MESSAGE_SIZE, "Bookmark %d unset", c-1);
+			print_message(msg);
 			break;
 		case GOTOBOOKMARK_A:
 			if (! (b->bookmark_mask & (1 << c))) return BOOKMARK_NOT_SET;
@@ -275,10 +330,10 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		if (c < 0 && (c = request_number("Column", b->cur_x + b->win_x + 1))<0) return NUMERIC_ERROR(c);
 		goto_column(b, c ? --c : 0);
 		return OK;
-			
-	case INSERTSTRING_A: {
+	
+	case INSERTSTRING_A:
 		/* Since we are going to call another action, we do not want to record this insertion twice. */
-		int recording= b->recording;
+		recording= b->recording;
 		b->recording = 0;
 		error = ERROR;
 		if (p || (p = request_string("String", NULL, FALSE, FALSE, b->encoding == ENC_UTF8 || b->encoding == ENC_ASCII && b->opt.utf8auto))) {
@@ -299,8 +354,32 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		}
 		b->recording = recording;
 		return error;	
-	}
-		
+
+	case TABS_A:
+		SET_USER_FLAG(b, c, opt.tabs);
+		return OK;
+
+	case INSERTTAB_A:
+		recording = b->recording;
+		b->recording = 0;
+		NORMALIZE(c);
+		start_undo_chain(b);
+		if (b->opt.tabs) {
+			while (c-- > 0) {
+				error = do_action(b, INSERTCHAR_A, '\t', NULL);
+			}
+		}
+		else {
+			while (c-- > 0) {
+				do {
+					error = do_action(b, INSERTCHAR_A, ' ', NULL);
+				} while (b->opt.tab_size && (b->win_x + b->cur_x) % b->opt.tab_size);
+			}
+		}
+		end_undo_chain(b);
+		b->recording = recording;
+		return error;
+
 	case INSERTCHAR_A: {
 
 		static int last_inserted_char = ' ';
@@ -330,7 +409,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		if (b->cur_pos > b->cur_line_desc->line_len) {
 			/* We insert spaces to reach the insertion position. */
 			insert_spaces(b, b->cur_line_desc, b->cur_line, b->cur_line_desc->line_len, b->cur_pos - b->cur_line_desc->line_len);
-			if (b->syn) update_line(b, b->cur_y, TRUE);
+			if (b->syn) update_line(b, b->cur_y, TRUE, TRUE);
 		}
 		
 		insert_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos, c);
@@ -355,27 +434,27 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		if (error == ERROR) {
 			assert_buffer_content(b);
 			/* No word wrap. */
-			if (b->syn) update_line(b, b->cur_y, FALSE);
+			if (b->syn) update_line(b, b->cur_y, TRUE, FALSE);
 			assert_buffer_content(b);
 		}		
 		else {		
 			/* Fixes in case of word wrapping. */
 			int wont_scroll = b->win_x == 0;
 			int a = 0;
-			if (b->syn) update_line(b, b->cur_y, TRUE);
+			if (b->syn) update_line(b, b->cur_y, FALSE, TRUE);
 			else update_partial_line(b, b->cur_y, calc_width(b->cur_line_desc, b->cur_line_desc->line_len, b->opt.tab_size, b->encoding) - b->win_x, FALSE, FALSE);	
 
 			need_attr_update = FALSE;
 			/* Poke the correct state into the next line. */
 			if (b->syn) ((line_desc *)b->cur_line_desc->ld_node.next)->highlight_state = b->next_state;
 
-			if (b->opt.auto_indent) a = auto_indent_line(b, b->cur_line + 1, (line_desc *)b->cur_line_desc->ld_node.next);
+			if (b->opt.auto_indent) a = auto_indent_line(b, b->cur_line + 1, (line_desc *)b->cur_line_desc->ld_node.next, INT_MAX);
 			move_to_sol(b);
 			line_down(b);
 			goto_pos(b, error + a);
                                                                
 			if (wont_scroll) {
-				if (b->cur_line == b->num_lines - 1) update_line(b, b->cur_y, FALSE);
+				if (b->cur_line == b->num_lines - 1) update_line(b, b->cur_y, FALSE, FALSE);
 				else scroll_window(b, b->cur_y, 1);
 			}
 
@@ -395,15 +474,71 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 		NORMALIZE(c);
 		
+		start_undo_chain(b);
 		for(i = 0; i < c && !stop; i++) {
 			if (a == BACKSPACE_A) {
-				if (b->win_x + b->cur_x == 0 && b->cur_line == 0) return ERROR;
-				else char_left(b);
-			}	
-			/* TODO: Here an additional condition line_len > 0 was present. Is it useful? It's just for deleting outside of lines. */
+				if (b->cur_pos == 0) {
+					if (b->cur_line == 0) {
+						/* Start of buffer. We just return an error. */
+						end_undo_chain(b);
+						return ERROR;
+					}
+					/* We turn a backspace at the start of a line into a delete 
+						at the end of the previous line. */
+					char_left(b);
+				}
+				else {
+					if (!b->opt.tabs && (b->win_x + b->cur_x) % b->opt.tab_size == 0
+						&& (b->cur_pos > b->cur_line_desc->line_len || b->cur_line_desc->line[b->cur_pos - 1] == ' ')) {
+						/* We are deleting one or more spaces from a tabbing position. We go left until the
+						previous tabbing, or when spaces end. */
+						do char_left(b); while((b->win_x + b->cur_x) % b->opt.tab_size != 0 
+															&& (b->cur_pos > b->cur_line_desc->line_len || b->cur_line_desc->line[b->cur_pos - 1] == ' '));
+					}
+					else char_left(b);
+					/* If we are not over text, we are in free form mode; the backspace
+						is turned into moving to the left. */
+					if (b->cur_pos >= b->cur_line_desc->line_len) continue;
+				}
+			}
+			
+			/* From here, we just implement a delete. */
+			
+			if (!b->opt.tabs && b->cur_pos < b->cur_line_desc->line_len && b->cur_line_desc->line[b->cur_pos] == ' ' && 
+				((b->win_x + b->cur_x) % b->opt.tab_size == 0 || b->cur_line_desc->line[b->cur_pos - 1] != ' ')) {
+				col = 0;
+				do col++; while((b->win_x + b->cur_x + col) % b->opt.tab_size != 0 
+					&& b->cur_pos + col < b->cur_line_desc->line_len && b->cur_line_desc->line[b->cur_pos + col] == ' ');
+				/* We are positioned at the start of the block of col spaces. If there is at most
+					one character to delete, we can just go on. Otherwise, we replace the block with a 
+					TAB, doing some magick to keep everything in sync. */
+				if (col > 1 && (b->win_x + b->cur_x + col) % b->opt.tab_size == 0) {
+					if (b->syn) {
+						freeze_attributes(b, b->cur_line_desc);
+						memmove(b->attr_buf + b->cur_pos + 1, b->attr_buf + b->cur_pos + col, b->attr_len - (b->cur_pos + col));
+						b->attr_buf[b->cur_pos] = -1;
+						b->attr_len -= (col - 1);
+					}
+					delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, col);
+					insert_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos, '\t');
+					if (b->syn) update_partial_line(b, b->cur_y, b->cur_x, TRUE, TRUE);	
+				}
+			}
+
 			if (b->cur_pos > b->cur_line_desc->line_len) {
-				/* We are not over text. */
-				continue;
+				col = b->win_x + b->cur_x;
+				/* We are not over text; we must be in FreeForm mode.
+				   We're deleting past the end of the line, so if we aren't on the last line
+				   we need to pad this line with space up to col, then fall through to the
+				   delete_one_char() below. */
+				if (b->cur_line_desc->ld_node.next->next == NULL) continue;
+				if (b->cur_line_desc->line_len == 0) {
+					auto_indent_line(b, b->cur_line, b->cur_line_desc, col);
+					resync_pos(b);
+				}
+				/* We need spaces if the line was not empty, or if we were sitting in the middle of a TAB. */
+				insert_spaces(b, b->cur_line_desc, b->cur_line, b->cur_line_desc->line_len, col - calc_width(b->cur_line_desc, b->cur_line_desc->line_len, b->opt.tab_size, b->encoding));
+				if (b->syn) freeze_attributes(b, b->cur_line_desc);
 			}
 
 			if (b->syn && b->attr_len < 0) freeze_attributes(b, b->cur_line_desc);
@@ -414,7 +549,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 				delete_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos);
 				
 				update_deleted_char(b, old_char, b->cur_line_desc, b->cur_pos, b->cur_char, b->cur_y, b->cur_x);	
-				if (b->syn) update_line(b, b->cur_y, TRUE);
+				if (b->syn) update_line(b, b->cur_y, TRUE, TRUE);
 			}
 			else {
 				/* Here we handle the case in which two lines are joined. Note that if the first line is empty,
@@ -426,7 +561,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 				
 				if (b->syn) {
 					b->next_state = parse(b->syn, b->cur_line_desc, b->cur_line_desc->highlight_state, b->encoding == ENC_UTF8); 
-					update_line(b, b->cur_y, TRUE);
+					update_line(b, b->cur_y, FALSE, TRUE);
 				}	
 				else update_partial_line(b, b->cur_y, b->cur_x, TRUE, FALSE);
 					
@@ -434,6 +569,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			}			
 		}
 		need_attr_update = TRUE;
+		end_undo_chain(b);
 		return stop ? STOPPED : 0;
 
 	case INSERTLINE_A:
@@ -444,8 +580,9 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			if (b->syn && b->attr_len < 0) freeze_attributes(b, b->cur_line_desc);
 
 			if (insert_one_line(b, b->cur_line_desc, b->cur_line, b->cur_pos > b->cur_line_desc->line_len ? b->cur_line_desc->line_len : b->cur_pos) == OK) {		
-				/* TODO: check that update agress with word wrapping in INSERTCHAR. */
+
 				if (b->win_x) {
+					int a = -1;
 					/* If b->win_x is nonzero, the move_to_sol() call will
 					refresh the entire video, so we shouldn't do anything. However, we 
 					must poke into the next line initial state the correct state. */
@@ -453,12 +590,17 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 						freeze_attributes(b, b->cur_line_desc);
 						((line_desc *)b->cur_line_desc->ld_node.next)->highlight_state = b->next_state;
 					}
+
+					assert(b->cur_line_desc->ld_node.next->next != NULL);
+					if (b->opt.auto_indent) a = auto_indent_line(b, b->cur_line + 1, (line_desc *)b->cur_line_desc->ld_node.next, INT_MAX);
+
 					move_to_sol(b);
 					line_down(b);
+					if (a != -1) goto_pos(b, a);					
 				}
 				else {
 					int a = -1;
-					if (b->syn) update_line(b, b->cur_y, TRUE);
+					if (b->syn) update_line(b, b->cur_y, FALSE, TRUE);
 					else update_partial_line(b, b->cur_y, b->cur_x, FALSE, FALSE);
 					/* We need to avoid updates until we fix the next line. */
 					need_attr_update = FALSE;
@@ -466,13 +608,13 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 					if (b->syn) ((line_desc *)b->cur_line_desc->ld_node.next)->highlight_state = b->next_state;
 
 					assert(b->cur_line_desc->ld_node.next->next != NULL);
-					if (b->opt.auto_indent) a = auto_indent_line(b, b->cur_line + 1, (line_desc *)b->cur_line_desc->ld_node.next);
+					if (b->opt.auto_indent) a = auto_indent_line(b, b->cur_line + 1, (line_desc *)b->cur_line_desc->ld_node.next, INT_MAX);
 
 					move_to_sol(b);
 					line_down(b);
 					if (a != -1) goto_pos(b, a);					
 
-					if (b->cur_line == b->num_lines - 1) update_line(b, b->cur_y, FALSE);
+					if (b->cur_line == b->num_lines - 1) update_line(b, b->cur_y, FALSE, FALSE);
 					else scroll_window(b, b->cur_y, 1);
 
 					need_attr_update = TRUE;
@@ -495,7 +637,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		}
 		end_undo_chain(b);
 		if (b->syn) {
-			update_line(b, b->cur_y, FALSE);
+			update_line(b, b->cur_y, FALSE, FALSE);
 			need_attr_update = TRUE;
 		}
 		resync_pos(b);
@@ -524,7 +666,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 						We perform a differential update so that if we undelete in the middle of
 						a line we avoid to rewrite the part up to b->cur_pos. */
 					b->attr_len = b->cur_pos;
-					update_line(b, b->cur_y, TRUE);
+					update_line(b, b->cur_y, FALSE, TRUE);
 					next_line_state = b->next_state;
 				}
 				else update_partial_line(b, b->cur_y, b->cur_x, FALSE, FALSE);
@@ -550,7 +692,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
 		if (b->syn && b->attr_len < 0) freeze_attributes(b, b->cur_line_desc);
 		delete_to_eol(b, b->cur_line_desc, b->cur_line, b->cur_pos);
-		if (b->syn) update_line(b, b->cur_y, TRUE);
+		if (b->syn) update_line(b, b->cur_y, FALSE, TRUE);
 		else update_partial_line(b, b->cur_y, b->cur_x, FALSE, FALSE);
 		need_attr_update = TRUE;
 
@@ -560,13 +702,19 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		p = str_dup(b->filename);
 
 	case SAVEAS_A:
-		if (p || (p = request_file(b, "Filename", b->filename))) {
+		if (p || (q = p = request_file(b, "Filename", b->filename))) {
 			print_info(SAVING);
 
 			error = save_buffer_to_file(b, p);
 
 			if (!print_error(error)) {
+				const int load_syntax = b->filename == NULL || ! same_str(extension(p), extension(b->filename));
 				change_filename(b, p);
+				if (load_syntax && extension(p)) {
+					load_syntax_by_name(b, extension(p));
+					load_auto_prefs(b, extension(p));
+					reset_window();
+				}
 				print_info(SAVED);
 			}
 			else {
@@ -577,8 +725,18 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		b->undo.last_save_step = b->undo.cur_step;
 		return OK;
 
+	case KEYCODE_A:
+		print_message(info_msg[PRESS_A_KEY]);
+		c = get_key_code();
+		i = CHAR_CLASS(c);
+		col = (c < 0) ? -c-1 : c;
+		snprintf(msg, MAX_MESSAGE_SIZE, "Key Code: 0x%02x,  Input Class: %s,  Assigned Command: %s", col, input_class_names[i],
+		         (key_binding[col] && key_binding[col][0]) ? key_binding[col] : "(none)" );
+		print_message(msg);
+		return OK;
+
 	case CLEAR_A:
-		if ((b->is_modified) && !request_response(b, "This document is not saved; are you sure?", FALSE)) return ERROR;
+		if ((b->is_modified) && !request_response(b, info_msg[THIS_DOCUMENT_NOT_SAVED], FALSE)) return ERROR;
 		clear_buffer(b);
 		reset_window();
 		return OK;
@@ -588,13 +746,18 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		reset_window();
 
 	case OPEN_A:
-		if ((b->is_modified) && !request_response(b, "This document is not saved; are you sure?", FALSE)) return ERROR;
+		if ((b->is_modified) && !request_response(b, info_msg[THIS_DOCUMENT_NOT_SAVED], FALSE)) {
+			if (a == OPENNEW_A) do_action(b, CLOSEDOC_A, 1, NULL);
+			return ERROR;
+		}
 
 		if (p || (p = request_file(b, "Filename", b->filename))) {
+			static int dprompt = 0; /* Set to true if we ever respond 'yes' to the prompt. */
 
 			buffer *dup = get_buffer_named(p);
 
-			if (!dup || dup == b || request_response(b, "There is another document with the same name; are you sure?", FALSE)) {
+			/* 'c' -- flag meaning "Don't prompt if we've ever responded 'yes'." */
+			if (!dup || dup == b || (dprompt && !c ) || (dprompt = request_response(b, info_msg[SAME_NAME], FALSE))) {
 				b->syn = NULL; /* So that autoprefs will load the right syntax. */
 				if (b->opt.auto_prefs && extension(p)) load_auto_prefs(b, extension(p));
 				error = load_file_in_buffer(b, p);
@@ -605,6 +768,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			}
 			free(p);
 		}
+		if (a == OPENNEW_A) do_action(b, CLOSEDOC_A, 1, NULL);
 		return ERROR;
 
 	case ABOUT_A:
@@ -632,7 +796,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			free(b->find_string);
 			b->find_string = p;
 			b->find_string_changed = 1;
-			print_error(error = (a == FIND_A ? find : find_regexp)(b, NULL, 0, FALSE));
+			print_error(error = (a == FIND_A ? find : find_regexp)(b, NULL, FALSE));
 		}
 
 		b->last_was_replace = 0;
@@ -666,10 +830,10 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 			if (p || (p = request_string(b->last_was_regexp ? "Replace RegExp" : "Replace", b->replace_string, TRUE, FALSE, b->encoding == ENC_UTF8 || b->encoding == ENC_ASCII && b->opt.utf8auto))) {
 				const encoding_type replace_encoding = detect_encoding(p, strlen(p));
-				int dir = b->opt.search_back ? -1 : 1, first_search = TRUE, num_replace = 0;
+				int first_search = TRUE, num_replace = 0;
 
 				if (replace_encoding != ENC_ASCII && b->encoding != ENC_ASCII && replace_encoding != b->encoding ||
-					 search_encoding != ENC_ASCII && replace_encoding != ENC_ASCII && search_encoding != replace_encoding) {
+					search_encoding != ENC_ASCII && replace_encoding != ENC_ASCII && search_encoding != replace_encoding) {
 					free(p);
 					return INCOMPATIBLE_REPLACE_STRING_ENCODING;
 				}
@@ -683,12 +847,11 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 				if (a == REPLACEALL_A) start_undo_chain(b);
 
 				while(!stop && 
-						!(error = (b->last_was_regexp ? find_regexp : find)(b, NULL, dir, !first_search && a != REPLACEALL_A && c != 'A' && c != 'Y')) &&
-						!((line_desc *)((line_desc *)b->cur_line_desc->ld_node.next)->ld_node.next == NULL && b->cur_pos == b->cur_line_desc->line_len)) {
+						!(error = (b->last_was_regexp ? find_regexp : find)(b, NULL, !first_search && a != REPLACEALL_A && c != 'A' && c != 'Y'))) {
 
 					if (c != 'A' && a != REPLACEALL_A && a != REPLACEONCE_A) {
 						refresh_window(b);
-						c = request_char(b, dir > 0 ? "Replace (Yes/No/Last/All/Quit/Backward)" : "Replace (Yes/No/Last/All/Quit/Forward)", 'n');
+						c = request_char(b, b->opt.search_back ? "Replace (Yes/No/Last/All/Quit/Forward)" : "Replace (Yes/No/Last/All/Quit/Backward)", 'n');
 						if (c == 'Q') break;
 						if (c == 'A') start_undo_chain(b);
 					}
@@ -699,21 +862,28 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 						if (b->last_was_regexp) error = replace_regexp(b, p);
 						else error = replace(b, strlen(b->find_string), p);
-						update_line(b, b->cur_y, FALSE);
-						if (b->syn) {
-							need_attr_update = TRUE;
-							update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
-						}
 						
+						if (!error) {
+							update_line(b, b->cur_y, FALSE, FALSE);
+							if (b->syn) {
+								need_attr_update = TRUE;
+								update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
+							}
+						
+							num_replace++;
+
+							if (last_replace_empty_match)
+								if (b->opt.search_back) error = char_left(cur_buffer);
+								else error = char_right(cur_buffer);
+						}					
+	
 						if (print_error(error)) {
 							if (a == REPLACEALL_A || c == 'A') end_undo_chain(b);
 							return ERROR;
 						}
-						num_replace++;
 					}
 
 					if (c == 'B' && !(b->opt.search_back) || c == 'F' && (b->opt.search_back)) {
-						dir = -dir;
 						b->opt.search_back = !b->opt.search_back;
 						b->find_string_changed = 1;
 					}
@@ -725,6 +895,10 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 				if (a == REPLACEALL_A || c == 'A') end_undo_chain(b);
 
+				if (num_replace) {
+					snprintf(msg, MAX_MESSAGE_SIZE, "%d replacement%s made.", num_replace, num_replace > 1 ? "s" : "");
+					print_message(msg);
+				}
 				if (stop) return STOPPED;
 
 				if ((c != 'A' && a != REPLACEALL_A || first_search) && error || error != NOT_FOUND) {
@@ -748,18 +922,30 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			if (b->last_was_replace) {
 				const encoding_type replace_encoding = detect_encoding(b->replace_string, strlen(b->replace_string));
 				if (replace_encoding != ENC_ASCII && b->encoding != ENC_ASCII && replace_encoding != b->encoding ||
-					 search_encoding != ENC_ASCII && replace_encoding != ENC_ASCII && search_encoding != replace_encoding)
+					search_encoding != ENC_ASCII && replace_encoding != ENC_ASCII && search_encoding != replace_encoding)
 					return INCOMPATIBLE_REPLACE_STRING_ENCODING;
 			}
 
 			NORMALIZE(c);
 
 			for(i = 0; i < c; i++) {
-				if (!print_error((b->last_was_regexp ? find_regexp : find)(b, NULL, 0, !b->last_was_replace))) {
+				if (!print_error((b->last_was_regexp ? find_regexp : find)(b, NULL, !b->last_was_replace))) {
 					if (b->last_was_replace) {
 						if (b->last_was_regexp) error = replace_regexp(b, b->replace_string);
 						else error = replace(b, strlen(b->find_string), b->replace_string);
-						update_line(b, b->cur_y, FALSE);
+
+						if (! error) {
+							update_line(b, b->cur_y, FALSE, FALSE);
+							if (b->syn) {
+								need_attr_update = TRUE;
+								update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
+							}
+						
+							if (last_replace_empty_match)
+								if (b->opt.search_back) error = char_left(cur_buffer);
+								else error = char_right(cur_buffer);
+						}	
+
 						if (print_error(error)) {
 							return_code = ERROR;
 							break;
@@ -841,7 +1027,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return OK;
 
 	case STATUSBAR_A:
-		SET_USER_FLAG(b, c, opt.status_bar);
+		SET_GLOBAL_FLAG(c, status_bar);
 		reset_status_bar();
 		return OK;
 
@@ -851,7 +1037,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return OK;
 
 	case FASTGUI_A:
-		SET_USER_FLAG(b, c, opt.fast_gui);
+		SET_GLOBAL_FLAG(c, fast_gui);
 		reset_status_bar();
 		return OK;
 
@@ -868,7 +1054,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return OK;
 
 	case VERBOSEMACROS_A:
-		SET_USER_FLAG(b, c, opt.verbose_macros);
+		SET_GLOBAL_FLAG(c, verbose_macros);
 		return OK;
 
 	case AUTOPREFS_A:
@@ -883,6 +1069,10 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		SET_USER_FLAG(b, c, opt.no_file_req);
 		return OK;
 
+	case REQUESTORDER_A:
+		SET_GLOBAL_FLAG(c, req_order);
+		return OK;
+
 	case UTF8AUTO_A:
 		SET_USER_FLAG(b, c, opt.utf8auto);
 		return OK;
@@ -936,13 +1126,13 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return OK;
 
 	case RECORD_A:
-		i = b->recording;
+		recording = b->recording;
 		SET_USER_FLAG(b, c, recording);
-		if (b->recording && !i) {
+		if (b->recording && !recording) {
 			b->cur_macro = reset_stream(b->cur_macro);
 			print_message(info_msg[STARTING_MACRO_RECORDING]);
 		}
-		else if (!b->recording && i) print_message(info_msg[MACRO_RECORDING_COMPLETED]);
+		else if (!b->recording && recording) print_message(info_msg[MACRO_RECORDING_COMPLETED]);
 		return OK;
 
 	case PLAY_A:
@@ -958,7 +1148,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 	case SAVEMACRO_A:
 		if (p || (p = request_file(b, "Macro Name", NULL))) {
 			print_info(SAVING);
-			optimize_macro(b->cur_macro, b->opt.verbose_macros);
+			optimize_macro(b->cur_macro, verbose_macros);
 			if ((error = print_error(save_stream(b->cur_macro, p, b->is_CRLF, FALSE))) == OK) print_info(SAVED);
 			free(p);
 			return error ? ERROR : 0;
@@ -993,7 +1183,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return OK;
 
 	case CLOSEDOC_A: 
-		if ((b->is_modified) && !request_response(b, "This document is not saved; are you sure?", FALSE)) return ERROR;
+		if ((b->is_modified) && !request_response(b, info_msg[THIS_DOCUMENT_NOT_SAVED], FALSE)) return ERROR;
 		if (!delete_buffer()) {
 			close_history();
 			unset_interactive_mode();
@@ -1038,7 +1228,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 	case MARKVERT_A:
 		SET_USER_FLAG(b, c, marking);
 		if (!b->marking) return(OK);
-		print_message(info_msg[BLOCK_START_MARKED]);
+		print_message(info_msg[a==MARK_A ? BLOCK_START_MARKED : VERTICAL_BLOCK_START_MARKED]);
 		b->mark_is_vertical = (a == MARKVERT_A);
 		b->block_start_line = b->cur_line;
 		b->block_start_col = b->win_x + b->cur_x;
@@ -1230,7 +1420,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		for(i = 0; i < c && !(error = center(b)) && !stop; i++) {
 			need_attr_update = TRUE;
 			b->attr_len = -1;
-			update_line(b, b->cur_y, FALSE);
+			update_line(b, b->cur_y, FALSE, FALSE);
 			move_to_sol(b);
 			if (line_down(b) != OK) break;
 		}
@@ -1279,10 +1469,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		if (p || (p = request_string("Syntax",  b->syn ? b->syn->name : NULL, TRUE, FALSE, b->encoding == ENC_UTF8 || b->encoding == ENC_ASCII && b->opt.utf8auto))) {
 			if (!strcmp(p, "*")) b->syn = NULL;
 			else error = print_error(load_syntax_by_name(b, p));
-			if (error == OK) {
-				reset_syntax_states(b);
-				reset_window();
-			}
+			if (error == OK) reset_window();
 			free(p);
 			return error ? ERROR : OK;
 		}
@@ -1330,12 +1517,48 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return OK;
 
 	case SUSPEND_A:
-#ifndef _AMIGA
 		stop_ne();
-#endif
 		keep_cursor_on_screen(cur_buffer);
 		return OK;
 
+	case AUTOCOMPLETE_A:
+		/* Since we are going to call other actions (INSERTSTRING_A and DELETEPREVWORD_A),
+			we do not want to record this insertion twice. Also, we are counting on 
+			INSERTSTRING_A to handle character encoding issues. */
+		recording = b->recording;
+		i = b->cur_pos;
+		
+		if ( !p ) { /* no prefix given; find one left of the cursor. */
+			if (i && i <= b->cur_line_desc->line_len) {
+				i = prev_pos(b->cur_line_desc->line, i, b->encoding);
+				while (i && ne_isword(c = get_char(&b->cur_line_desc->line[i], b->encoding), b->encoding))
+					i = prev_pos(b->cur_line_desc->line, i, b->encoding);
+				if (! ne_isword(c = get_char(&b->cur_line_desc->line[i], b->encoding), b->encoding))
+					i = next_pos(b->cur_line_desc->line, i, b->encoding);
+				p = malloc(b->cur_pos - i + 1);
+				if (!p) return OUT_OF_MEMORY;
+				strncpy(p, &b->cur_line_desc->line[i], b->cur_pos - i);
+			} 
+			else p = malloc(1); /* no prefix left of the cursor; we'll find _all_ word strings! */
+			
+			p[b->cur_pos - i] = 0;
+		}	
+
+		snprintf(msg, MAX_MESSAGE_SIZE, "AutoComplete: prefix \"%s\"", p);
+			
+		if (p = autocomplete(p, msg, TRUE, &col)) {
+			b->recording = 0;
+			start_undo_chain(b);
+			if (i >= b->cur_pos || (error = do_action(b, DELETEPREVWORD_A, 1, NULL)) == OK) error = do_action(b, INSERTSTRING_A, 0, p);
+			end_undo_chain(b);
+			b->recording = recording;
+			print_message(info_msg[col]);
+		}
+		else if (stop) error = STOPPED;
+		else if (col == AUTOCOMPLETE_NO_MATCH) print_message(info_msg[AUTOCOMPLETE_NO_MATCH]);
+		
+		return print_error(error) ? ERROR : 0;
+
 	default:
 		if (p) free(p);
 		return OK;
