@@ -1,7 +1,7 @@
 /* Buffer handling functions, including allocation, deallocation, and I/O.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -185,9 +185,8 @@ buffer *alloc_buffer(const buffer * const cur_b) {
 		b->cur_macro = alloc_char_stream(0);
 		b->opt.tab_size = 8;
 
-		b->opt.status_bar     =
 		b->opt.insert         =
-		b->opt.verbose_macros =
+		b->opt.tabs           =
 		b->opt.do_undo        =
 		b->opt.auto_prefs     = 1;
 
@@ -199,17 +198,15 @@ buffer *alloc_buffer(const buffer * const cur_b) {
 
 			b->opt.cur_clip       = cur_b->opt.cur_clip;
 			b->opt.tab_size       = cur_b->opt.tab_size;
+			b->opt.tabs           = cur_b->opt.tabs;
 			b->opt.right_margin   = cur_b->opt.right_margin;
 
 			b->opt.free_form      = cur_b->opt.free_form;
-			b->opt.status_bar     = cur_b->opt.status_bar;
 			b->opt.hex_code       = cur_b->opt.hex_code;
-			b->opt.fast_gui       = cur_b->opt.fast_gui;
 			b->opt.word_wrap      = cur_b->opt.word_wrap;
 			b->opt.auto_indent    = cur_b->opt.auto_indent;
 			b->opt.preserve_cr    = cur_b->opt.preserve_cr;
 
-			b->opt.verbose_macros = cur_b->opt.verbose_macros;
 			b->opt.do_undo        = cur_b->opt.do_undo;
 			b->opt.auto_prefs     = cur_b->opt.auto_prefs;
 			b->opt.no_file_req    = cur_b->opt.no_file_req;
