@@ -1,7 +1,7 @@
 /* Command table manipulation functions and vectors.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -24,18 +24,13 @@
 #include "help.h"
 #include "hash.h"
 
+#undef TABSIZE
+
 /* The standard macro descriptor allocation dimension. */
 
 #define STD_MACRO_DESC_SIZE	1024
 
 
-/* The maximum width for a command (used for displaying the command
-names with the string requester). It allows four columns on an 80x25 screen.
-*/
-
-#define MAX_COMMAND_WIDTH		18
-
-
 /* This structure represents a command. It includes a long and a short name,
 a NULL-terminated vector of help strings (of specified length) and some flags
 which are related to the syntax and the semantics of the arguments. */
@@ -62,13 +57,13 @@ typedef struct {
 #define NAHL(x) x ## _NAME, x ##_ABBREV, x ## _HELP, HELP_LEN(x)
 
 /* This is the command vector. Note that the command names come from names.h,
-   and the help names come from help.h. */
+   and the help names come from help.h. This must be kept sorted. */
 
 static const command commands[ACTION_COUNT] = {
-
 	{ NAHL(ABOUT         ), NO_ARGS                                                               },
 	{ NAHL(ADJUSTVIEW    ),           ARG_IS_STRING                                               },
 	{ NAHL(ALERT         ), NO_ARGS                                                               },
+	{ NAHL(AUTOCOMPLETE  ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(AUTOINDENT    ),                           IS_OPTION                                   },
 	{ NAHL(AUTOPREFS     ),                           IS_OPTION                                   },
 	{ NAHL(BACKSPACE     ),0                                                                      },
@@ -86,6 +81,8 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(DELETECHAR    ),0                                                                      },
 	{ NAHL(DELETEEOL     ), NO_ARGS                                                               },
 	{ NAHL(DELETELINE    ),0                                                                      },
+	{ NAHL(DELETENEXTWORD),0                                                                      },
+	{ NAHL(DELETEPREVWORD),0                                                                      },
 	{ NAHL(DOUNDO        ),                           IS_OPTION                                   },
 	{ NAHL(ERASE         ),0                                                                      },
 	{ NAHL(ESCAPE        ),                                       DO_NOT_RECORD                   },
@@ -108,6 +105,8 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(INSERTCHAR    ),0                                                                      },
 	{ NAHL(INSERTLINE    ),0                                                                      },
 	{ NAHL(INSERTSTRING  ),           ARG_IS_STRING                                               },
+	{ NAHL(INSERTTAB     ),0                                                                      },
+	{ NAHL(KEYCODE       ),                                       DO_NOT_RECORD                   },
 	{ NAHL(LINEDOWN      ),0                                                                      },
 	{ NAHL(LINEUP        ),0                                                                      },
 	{ NAHL(LOADAUTOPREFS ), NO_ARGS                                                               },
@@ -159,6 +158,7 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(REPLACE       ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(REPLACEALL    ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(REPLACEONCE   ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
+	{ NAHL(REQUESTORDER  ),                           IS_OPTION                                   },
 	{ NAHL(RIGHTMARGIN   ),                           IS_OPTION                                   },
 	{ NAHL(SAVE          ), NO_ARGS                                                               },
 	{ NAHL(SAVEAS        ),           ARG_IS_STRING                                               },
@@ -174,6 +174,7 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(SUSPEND       ), NO_ARGS                                                               },
 	{ NAHL(SYNTAX        ),           ARG_IS_STRING | IS_OPTION                                   },
 	{ NAHL(SYSTEM        ),           ARG_IS_STRING                                               },
+	{ NAHL(TABS          ),                           IS_OPTION                                   },
 	{ NAHL(TABSIZE       ),                           IS_OPTION                                   },
 	{ NAHL(THROUGH       ),           ARG_IS_STRING                                               },
 	{ NAHL(TOGGLESEOF    ), NO_ARGS                                                               },
@@ -640,11 +641,12 @@ void unload_macros(void) {
 void help(char *p) {
 
 	action a;
-	int i;
-	D(fprintf(stderr,"Help Called with parm %ld.\n",p);)
+	int j, i = 0;
+	D(fprintf(stderr,"Help Called with parm %p.\n",p);)
 	do {
-		if (p || (i = request_strings(command_names, ACTION_COUNT, MAX_COMMAND_WIDTH, FALSE)) >= 0) {
-         D(fprintf(stderr,"Help check #2: p=%lx, i=%d\n",p,i);)
+		print_message("Help: select Command and press Enter, or F1 or Escape or Escape-Escape");
+		if (p || (i = request_strings(command_names, ACTION_COUNT, i, MAX_COMMAND_WIDTH, FALSE)) >= 0) {
+         D(fprintf(stderr,"Help check #2: p=%p, i=%d\n",p,i);)
 			if (p) {
 				for(i = 0; i < strlen(p); i++) if (isasciispace((unsigned char)p[i])) break;
 
@@ -669,7 +671,9 @@ void help(char *p) {
 
 			assert(i >= 0 && i < ACTION_COUNT);
 
-			i = request_strings(commands[i].help, commands[i].help_len, ne_columns, FALSE);
+			print_message("Help: press Enter, or F1 or Escape or Escape-Escape");
+			if ((j = request_strings(commands[i].help, commands[i].help_len, 0, ne_columns, FALSE)) < 0 ) i = j;
 		}
 	} while(i >= 0);
+	draw_status_bar();
 }
