@@ -1,7 +1,7 @@
 /* Definition for debug statements and assertions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -22,26 +22,10 @@
 
 
 #include <stdlib.h>
-
-#ifndef NODEBUG
-
-void __assert(int x, char *expr, char *file, int line);
-
-#define assert(x) { __assert((x), #x, __FILE__, __LINE__); if(!(x)) abort();}
-
-#else
-
-#define assert(ignore)  ((void) 0)
-
-#endif
-
+#include <assert.h>
 
 #ifdef DEBUGPRINTF
-
 #define D(x) x
-
 #else
-
 #define D(x) ;
-
 #endif
