@@ -1,7 +1,7 @@
 /* Various editing functions such as word wrap, to upper, etc.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -43,14 +43,14 @@ static int to_something(buffer *b, int (to_first)(int), int (to_rest)(int)) {
 	if (b->cur_line == b->num_lines -1 && b->cur_pos >= b->cur_line_desc->line_len) return ERROR;
 
 	/* First of all, we search for the word start, if we're not over it. */
-	if (pos >= b->cur_line_desc->line_len || ne_ispunct(c = get_char(&b->cur_line_desc->line[pos], b->encoding), b->encoding) || ne_isspace(c, b->encoding)) search_word(b, 1);
+	if (pos >= b->cur_line_desc->line_len || !ne_isword(c = get_char(&b->cur_line_desc->line[pos], b->encoding), b->encoding)) search_word(b, 1);
 	x = b->cur_x; /* The original x position (to perform the line update). */
 
 	pos = b->cur_pos;
 	new_len = 0;
 	/* Then, we compute the word position extremes, length of the result (which
 		may change because of casing). */
-	while (pos < b->cur_line_desc->line_len && !(ne_ispunct(c = get_char(&b->cur_line_desc->line[pos], b->encoding), b->encoding) || ne_isspace(c, b->encoding))) {
+	while (pos < b->cur_line_desc->line_len && ne_isword(c = get_char(&b->cur_line_desc->line[pos], b->encoding), b->encoding)) {
 		new_c = new_len ? to_rest(c) : to_first(c);
 		changed |= (c != new_c);
 
@@ -73,7 +73,7 @@ static int to_something(buffer *b, int (to_first)(int), int (to_rest)(int)) {
 		pos = b->cur_pos;
 		new_len = 0;
 		/* Second pass: we actually build the transformed word. */
-		while (pos < b->cur_line_desc->line_len && !(ne_ispunct(c = get_char(&b->cur_line_desc->line[pos], b->encoding), b->encoding) || ne_isspace(c, b->encoding))) {
+		while (pos < b->cur_line_desc->line_len && ne_isword(c = get_char(&b->cur_line_desc->line[pos], b->encoding), b->encoding)) {
 			if (b->encoding == ENC_UTF8) new_len += utf8str(new_len ? to_rest(c) : to_first(c), word + new_len);
 			else {
 				word[new_len] = new_len ? to_rest(c) : to_first(c);
@@ -94,7 +94,7 @@ static int to_something(buffer *b, int (to_first)(int), int (to_rest)(int)) {
 	}
 
 	b->attr_len = -1;
-	update_line(b, b->cur_y, FALSE);
+	update_line(b, b->cur_y, FALSE, FALSE);
 	if (b->syn) {
 		need_attr_update = TRUE;
 		update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
@@ -328,7 +328,6 @@ static void trim_trailing_space(buffer * const b, line_desc *ld, const int line,
 	if (pos >= 0 && pos < ld->line_len) delete_stream(b, ld, line, pos, ld->line_len - pos);
 }
 
-
 /* is_part_of_paragraph() determines if the line ld refers to could be
 	considered part of a paragraph based on its leading spaces compared to
 	pa_space_len. If they are the same, is_part_of_paragraph() returns 1, and
@@ -541,20 +540,25 @@ int center(buffer * const b) {
 
 
 
-/* Indents a line of the amount of whitespace present on the previous line. The
-	number of inserted bytes is returned. */
+/* Indents a line of the amount of whitespace present on the previous line, stopping
+	at a given column (use INT_MAX for not stopping). The number of
+	inserted bytes is returned. */
 
-int auto_indent_line(buffer * const b, const int line, line_desc * const ld) {
+int auto_indent_line(buffer * const b, const int line, line_desc * const ld, const int up_to_col) {
 
 	line_desc * const prev_ld = (line_desc *)ld->ld_node.prev;
-	int pos = 0;
+	int pos = 0, col = 0, c;
 
 	assert(prev_ld->ld_node.prev != NULL);
 	assert_line_desc(prev_ld, b->encoding);
 
 	if (prev_ld->line_len == 0) return 0;
 
-	while(pos < prev_ld->line_len && ne_isspace(get_char(&prev_ld->line[pos], b->encoding), b->encoding)) pos = next_pos(prev_ld->line, pos, b->encoding);
+	while(pos < prev_ld->line_len && ne_isspace(c = get_char(&prev_ld->line[pos], b->encoding), b->encoding)) {
+		col += (c == '\t' ? b->opt.tab_size - col % b->opt.tab_size : 1);
+		if (col > up_to_col) break;
+		pos = next_pos(prev_ld->line, pos, b->encoding);
+	}
 	insert_stream(b, ld, line, 0, prev_ld->line, pos);
 	return pos;
 }
