@@ -1,7 +1,7 @@
 /* Error message vector.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -88,6 +88,15 @@ char *info_msg[INFO_COUNT] = {
 	"Saved.",
 	"Select file or press F1, Escape or Escape-Escape to enter a file name.",
 	"Start of block marked. Move to the end of block and request actions.",
+	"Start of vertical block marked. Move to the end of block and request actions.",
 	"Starting macro recording...",
-	"Macro recording completed."
+	"Macro recording completed.",
+	"Some documents have not been saved; are you sure?",
+	"Press a key to see ne's corresponding key code:",
+	"This document is not saved; are you sure?",
+	"There is another document with the same name; are you sure?",
+	"No matching words found.",
+	"Completed.",
+	"Partially completed.",
+	"Cancelled."
 };
