@@ -1,7 +1,7 @@
 /* Error index enum, and extern declaration of the error vector.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -108,8 +108,17 @@ enum info {
 	SAVED,
 	PRESSF1,
 	BLOCK_START_MARKED,
+	VERTICAL_BLOCK_START_MARKED,
 	STARTING_MACRO_RECORDING,
 	MACRO_RECORDING_COMPLETED,
+	SOME_DOCUMENTS_ARE_NOT_SAVED,
+	PRESS_A_KEY,
+	THIS_DOCUMENT_NOT_SAVED,
+	SAME_NAME,
+	AUTOCOMPLETE_NO_MATCH,
+	AUTOCOMPLETE_COMPLETED,
+	AUTOCOMPLETE_PARTIAL,
+	AUTOCOMPLETE_CANCELLED,
 	INFO_COUNT
 };
 
