@@ -1,7 +1,7 @@
 /* Help strings.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2005 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -27,8 +27,8 @@ const char * const ABOUT_HELP[5] = {
   "displays a simple information line about `ne' on the status bar.",
 };
 
-const char * const ADJUSTVIEW_HELP[20] = {
-  "Syntax: AdjustView [T|C|B|L|M|R]",
+const char * const ADJUSTVIEW_HELP[26] = {
+  "Syntax: AdjustView [T|M|B|L|C|R] [N]",
   "Abbreviation: AV",
   "",
   "shifts the view (text visible in the terminal window) horizontally or",
@@ -37,16 +37,22 @@ const char * const ADJUSTVIEW_HELP[20] = {
   "width of the current document. If called with no arguments `T' is",
   "assumed.",
   "",
-  "   `T', `C', and `B' cause vertical shifts so that the current line",
-  "becomes the top, center, or bottom-most visible line respectively.",
+  "   `T', `M', and `B' cause vertical shifts so that the current line",
+  "becomes the top, middle, or bottom-most visible line respectively.",
   "",
-  "   `L', `M', and `R' cause horizontal shifts, making the current column",
-  "the left-most, middle, or right-most visible positions.",
+  "   `L', `C', and `R' cause horizontal shifts, making the current column",
+  "the left-most, center, or right-most visible positions.",
+  "",
+  "   A optional number N immediately after `T', `B', `L', or `R' indicate",
+  "the number or rows or columns to shift the view toward the top, bottom,",
+  "left, or right of the window.",
   "",
   "   Horizontal and vertical adjustment specifications may be combined,",
   "so that for example `AdjustView TL' shifts the view so that the current",
   "position becomes the top left-most character on screen (within the",
-  "limits of the current TAB size).",
+  "limits of the current TAB size). Likewise, `AdjustView B3R5' shifts the",
+  "view three lines toward the bottom and five columns (excepting TAB",
+  "size) toward the right.",
 };
 
 const char * const ALERT_HELP[5] = {
@@ -56,6 +62,28 @@ const char * const ALERT_HELP[5] = {
   "beeps or flashes, depending on the value of the visual bell flag.",
 };
 
+const char * const AUTOCOMPLETE_HELP[20] = {
+  "Syntax: AutoComplete [PREFIX]",
+  "Abbreviation: AC",
+  "",
+  "attempts to extend the PREFIX using matching words from your open",
+  "documents, and inserts the extended text into your document. If the",
+  "PREFIX can be extended unambiguously, the matching text is immediately",
+  "inserted into your document. Otherwise, `ne' displays a selection of",
+  "all words in open documents that match PREFIX, and inserts the word you",
+  "select into the current document.  Matching words from the current",
+  "document display normally; those which only exist in other open",
+  "documents are bold and with a trailing asterisk. If no PREFIX is given",
+  "on the command line, or if `AutoComplete' is selected from the `Extras'",
+  "menu or using a keyboard shortcut, the word characters to the immediate",
+  "left of the cursor in the current document are used as the PREFIX. Note",
+  "that if no word characters are to the left of the cursor, or the PREFIX",
+  "given on the command line is an empty string (`\"\"'), then all words in",
+  "all your open documents are displayed. Prefix matches may be case",
+  "sensitive or not depending on the current document's `CaseSearch' flag",
+  "state. See `CaseSearch'.",
+};
+
 const char * const AUTOINDENT_HELP[22] = {
   "Syntax: AutoIndent [0|1]",
   "Abbreviation: AI",
@@ -271,6 +299,24 @@ const char * const DELETELINE_HELP[9] = {
   "respect to `InsertLine'.",
 };
 
+const char * const DELETENEXTWORD_HELP[6] = {
+  "Syntax: DeleteNextWord [N]",
+  "Abbreviation: DNW",
+  "",
+  "deletes text from the current position to the next word N times. If the",
+  "optional N argument is not specified, it is assumed to be one.",
+};
+
+const char * const DELETEPREVWORD_HELP[8] = {
+  "Syntax: DeletePrevWord [N]",
+  "Abbreviation: DPW",
+  "",
+  "deletes text from the current position to the first character of the",
+  "previous word N times. If the optional N argument is not specified, it",
+  "is assumed to be one (in which case, if the cursor is in the middle of a",
+  "word the effect is just to delete to the start of that word).",
+};
+
 const char * const DOUNDO_HELP[20] = {
   "Syntax: DoUndo [0|1]",
   "Abbreviation: DU",
@@ -351,7 +397,7 @@ const char * const EXIT_HELP[6] = {
   "cannot be saved, the action is suspended and an error message is issued.",
 };
 
-const char * const FASTGUI_HELP[15] = {
+const char * const FASTGUI_HELP[19] = {
   "Syntax: FastGUI [0|1]",
   "Abbreviation: FG",
   "",
@@ -366,6 +412,10 @@ const char * const FASTGUI_HELP[15] = {
   "   If you invoke `FastGUI' with no arguments, it will toggle the flag.",
   "If you specify 0 or 1, the flag will be set to false or true,",
   "respectively.",
+  "",
+  "   The `FastGUI' setting is saved in your `~/.ne/.default#ap' file when",
+  "you use the `SaveDefPrefs' command or the `Save Def Prefs' menu.  It is",
+  "not saved by the `SaveAutoPrefs' command.",
 };
 
 const char * const FIND_HELP[12] = {
@@ -396,7 +446,7 @@ const char * const FINDREGEXP_HELP[12] = {
   "on the input line, the default being the last pattern used.",
 };
 
-const char * const FLAGS_HELP[27] = {
+const char * const FLAGS_HELP[30] = {
   "Syntax: Flags",
   "Abbreviation: FLAG",
   "",
@@ -406,7 +456,7 @@ const char * const FLAGS_HELP[27] = {
   "     FLAG COMMAND      ABBR  DESCRIPTION",
   "       i   Insert        I    inserts new characters (vs. replacing)",
   "       a   AutoIndent    AI   aligns cursor under previous line after <Return>",
-  "       b   BackSearch    BS   searches search backward rather than forward",
+  "       b   SearchBack    SB   searches search backward rather than forward",
   "       c   CaseSearch    CS   searches are case sensitive",
   "       w   WordWrap      WW   breaks long lines as you type",
   "       f   FreeForm      FF   allows cursor to move beyond the end of lines",
@@ -414,6 +464,7 @@ const char * const FLAGS_HELP[27] = {
   "       v   VerboseMacros VM   record macros using use long command names",
   "       u   DoUndo        DU   record edits for later undoing",
   "       r   ReadOnly      RO   changes are not allowed",
+  "       t   Tabs          TAB  TAB key inserts TABs instead of spaces",
   "       B   Binary        B    affects file loading/saving",
   "       M   Mark          M    mark set for line-oriented block operations",
   "       V   MarkVert      MV   like mark, but block is rectangle",
@@ -423,6 +474,8 @@ const char * const FLAGS_HELP[27] = {
   "       *   Modified      MOD  document has been modified since last saved",
   "       @   UTF8IO        U8IO I/O (keyboard and terminal) are UTF-8 encoded",
   "     A/8/U UTF8          U8   the document encoding (ASCII, 8-bit or UTF-8)",
+  "   The `RequestOrder' flag's state is not indicated on the status bar.",
+  "See `RequestOrder'.",
 };
 
 const char * const FLASH_HELP[6] = {
@@ -541,14 +594,15 @@ const char * const INSERT_HELP[12] = {
   "flag is true.",
 };
 
-const char * const INSERTCHAR_HELP[20] = {
+const char * const INSERTCHAR_HELP[21] = {
   "Syntax: InsertChar [CODE]",
   "Abbreviation: IC",
   "",
   "inserts a character whose ASCII code is CODE at the current cursor",
-  "position.  CODE must always be different from 0. All the currently",
-  "active preferences options (insert, word wrapping, auto indent, et",
-  "cetera) are applied.",
+  "position. CODE can be either decimal, hexadecimal if preceded by `0x',",
+  "or octal if preceded by `0'. In any case, CODE must be different from",
+  "0. All the currently active preferences options (insert, word wrapping,",
+  "auto indent, et cetera) are applied.",
   "",
   "   If the optional argument CODE is not specified, you can enter it on",
   "the input line, the default being the last inserted character.",
@@ -586,6 +640,25 @@ const char * const INSERTSTRING_HELP[12] = {
   "read and edit. See `SaveMacro'.",
 };
 
+const char * const INSERTTAB_HELP[7] = {
+  "Syntax: InsertTab [N]",
+  "Abbreviation: IT",
+  "",
+  "inserts either N literal TAB characters or one or more spaces",
+  "sufficient to advance the current cursor position N tab stops depending",
+  "on the `Tabs' flag. See `Tabs', `TabSize'.",
+};
+
+const char * const KEYCODE_HELP[8] = {
+  "Syntax: KeyCode",
+  "Abbreviation: KC",
+  "",
+  "prompts you to press a key, and reports on the status line the key code",
+  "`ne' associates with that key. This can be useful while configuring",
+  "your `~/.ne/.keys' file. It also reports the input class for that key.",
+  "Input class codes are: ALPHA, COMMAND, RETURN, TAB, IGNORE, and INVALID.",
+};
+
 const char * const LINEDOWN_HELP[6] = {
   "Syntax: LineDown [N]",
   "Abbreviation: LD",
@@ -1020,11 +1093,11 @@ const char * const POPPREFS_HELP[24] = {
   "",
   "PopPrefs restores the following values from the preferences stack:",
   "",
-  "        AutoIndent   DoUndo    NoFileReq    StatusBar",
-  "        AutoPrefs    FastGUI   PreserveCR   TabSize",
-  "        Binary       FreeForm  ReadOnly     Turbo",
-  "        CaseSearch   HexCode   RightMargin  VerboseMacros",
-  "        ClipNumber   Insert    SearchBack   WordWrap",
+  "       AutoIndent   DoUndo      PreserveCR   Tabs",
+  "       AutoPrefs    FreeForm    ReadOnly     TabSize",
+  "       Binary       HexCode     RightMargin  UTF8Auto",
+  "       CaseSearch   Insert      SearchBack   VisualBell",
+  "       ClipNumber   NoFileReq   StatusBar    WordWrap",
 };
 
 const char * const PRESERVECR_HELP[15] = {
@@ -1090,11 +1163,11 @@ const char * const PUSHPREFS_HELP[24] = {
   "",
   "`PushPrefs' saves the following values on the preferences stack:",
   "",
-  "        AutoIndent   DoUndo    NoFileReq    StatusBar",
-  "        AutoPrefs    FastGUI   PreserveCR   TabSize",
-  "        Binary       FreeForm  ReadOnly     Turbo",
-  "        CaseSearch   HexCode   RightMargin  VerboseMacros",
-  "        ClipNumber   Insert    SearchBack   WordWrap",
+  "       AutoIndent   DoUndo      PreserveCR   Tabs",
+  "       AutoPrefs    FreeForm    ReadOnly     TabSize",
+  "       Binary       HexCode     RightMargin  UTF8Auto",
+  "       CaseSearch   Insert      SearchBack   VisualBell",
+  "       ClipNumber   NoFileReq   StatusBar    WordWrap",
 };
 
 const char * const QUIT_HELP[6] = {
@@ -1268,6 +1341,23 @@ const char * const REPLACEONCE_HELP[10] = {
   "the input line, the default being the last string used.",
 };
 
+const char * const REQUESTORDER_HELP[15] = {
+  "Syntax: RequestOrder [0|1]",
+  "Abbreviation: RQO",
+  "",
+  "sets the request order flag. When this flag is true, the requester",
+  "displays entries in column order. Otherwise entries are displayed by",
+  "rows.",
+  "",
+  "   If you invoke `RequestOrder' with no arguments, it will toggle the",
+  "flag. If you specify 0 or 1, the flag will be set to false or true,",
+  "respectively.",
+  "",
+  "   The `RequestOrder' setting is saved in your `~/.ne/.default#ap' file",
+  "when you use the `SaveDefPrefs' command or the `Save Def Prefs' menu.",
+  "It is not saved by the `SaveAutoPrefs' command.",
+};
+
 const char * const RIGHTMARGIN_HELP[12] = {
   "Syntax: RightMargin [N]",
   "Abbreviation: RM",
@@ -1424,7 +1514,7 @@ const char * const SETBOOKMARK_HELP[10] = {
   "bookmark.",
 };
 
-const char * const STATUSBAR_HELP[19] = {
+const char * const STATUSBAR_HELP[23] = {
   "Syntax: StatusBar [0|1]",
   "Abbreviation: ST",
   "",
@@ -1443,6 +1533,10 @@ const char * const STATUSBAR_HELP[19] = {
   "   If you invoke `StatusBar' with no arguments, it will toggle the",
   "flag. If you specify 0 or 1, the flag will be set to false or true,",
   "respectively.",
+  "",
+  "   The `StatusBar' setting is saved in your `~/.ne/.default#ap' file",
+  "when you use the `SaveDefPrefs' command or the `Save Def Prefs' menu.",
+  "It is not saved by the `SaveAutoPrefs' command.",
 };
 
 const char * const SUSPEND_HELP[6] = {
@@ -1453,7 +1547,7 @@ const char * const SUSPEND_HELP[6] = {
   "command `fg' is used to resume `ne'.",
 };
 
-const char * const SYNTAX_HELP[41] = {
+const char * const SYNTAX_HELP[42] = {
   "Syntax: Syntax [NAME|*]",
   "Abbreviation: SY",
   "",
@@ -1461,10 +1555,11 @@ const char * const SYNTAX_HELP[41] = {
   "accordingly.",
   "",
   "   If the optional NAME argument is not specified, you are prompted for",
-  "one. The special NAME * turns off syntax highlighting for the current",
-  "document. Otherwise, NAME must match a syntax definition either in your",
+  "one. The current one, if set, is suggested as the default. The special",
+  "NAME * turns off syntax highlighting for the current document.",
+  "Otherwise, NAME must match a syntax definition either in your",
   "`~/.ne/syntax' directory or in a directory named `syntax' inside `ne''s",
-  "global directory. Additionally, `ne' has a table mapping common",
+  "global directory.  Additionally, `ne' has a table mapping common",
   "suffixes to syntax names. If there is no syntax with a given name, `ne'",
   "will try to remap the name using the following table  (the string",
   "before the colon is the name of the syntax file):",
@@ -1484,13 +1579,13 @@ const char * const SYNTAX_HELP[41] = {
   "        pascal: p, pas",
   "        perl: pl, pm",
   "        ps: eps",
-  "        python: py",
+  "        python: py, sage",
   "        rexx: rex",
   "        ruby: rb",
   "        sh: bash, bash_login, bash_logout, bash_profile, bashrc, ksh,",
   "            profile, rc",
   "        skill: il",
-  "        tex: sty",
+  "        tex: latex, dtx, sty",
   "        troff: 1",
   "        verilog: v, vh, vhd",
   "        xml: xsd",
@@ -1508,6 +1603,32 @@ const char * const SYSTEM_HELP[10] = {
   "on the input line.",
 };
 
+const char * const TABS_HELP[24] = {
+  "Syntax: Tabs [0|1]",
+  "Abbreviation: TAB",
+  "",
+  "sets the `Tabs' flag. When this flag is true, the `InsertTab' command",
+  "will insert literal TAB characters. Otherwise it will insert enough",
+  "spaces to have the same visual effect.",
+  "",
+  "   In normal editing, the <Tab> key invokes the command \"`InsertTab'",
+  "1\". Unlike most others, the <Tab> key cannot be mapped to other",
+  "commands. Thus the `Tabs' flag provides the only customization `ne'",
+  "offers for the <Tab> key.",
+  "",
+  "   The `Tabs' flag also affects the action of the <BackSpace> and",
+  "<Delete> keys in one special circumstance. That is, if the normal",
+  "action of these keys would remove white space, and that white space",
+  "could be represented by a tab, then one tab's worth of space is",
+  "removed. In all other cases, the <BackSpace> and <Delete> keys perform",
+  "their normal function.",
+  "",
+  "   If you invoke `Tabs' with no arguments, it will toggle the flag. If",
+  "you specify 0 or 1, the flag will be set to false or true,",
+  "respectively. A lower case `t' will appear on the status bar if the",
+  "flag is true.",
+};
+
 const char * const TABSIZE_HELP[9] = {
   "Syntax: TabSize [SIZE]",
   "Abbreviation: TS",
@@ -1589,9 +1710,9 @@ const char * const TURBO_HELP[29] = {
   "Abbreviation: TUR",
   "",
   "sets the turbo parameter. Iterated actions and global replaces will",
-  "update at most STEPS line of the screen (or at most twice the number of",
-  "visible rows if STEPS is zero); then, update will be delayed to the end",
-  "of the action.",
+  "update at most STEPS lines of the screen (or at most twice the number",
+  "of visible rows if STEPS is zero); then, update will be delayed to the",
+  "end of the action.",
   "",
   "   This feature is most useful when massive operations (such as",
   "replacing thousands of occurrences of a pattern) have to be performed.",
@@ -1682,13 +1803,13 @@ const char * const UTF8_HELP[20] = {
   "you specify 0 or 1, the flag will be set to false or true,",
   "respectively. When you try to set this flag, the buffer will be checked",
   "for UTF-8 compliance, and you will get an error message in case of",
-  "failure. When you try to reset it, the buffer is set to ASCII or",
-  "ISO8859, depending on its content.  A `U' will appear on the status bar",
-  "if the flag is true. Alternatively, an `A' or an `8' will be displayed",
-  "to denote whether the buffer is composed exclusively by US-ASCII",
-  "characters, or also by other 8-bit characters (which are likely to be",
-  "ISO-8859 encoded). Note that each time this command modifies the buffer",
-  "encoding, it also resets the undo buffer.",
+  "failure. When you try to reset it, the buffer is set to ASCII or 8-bit,",
+  "depending on its content.  A `U' will appear on the status bar if the",
+  "flag is true. Alternatively, an `A' or an `8' will be displayed to",
+  "denote whether the buffer is composed exclusively by US-ASCII",
+  "characters, or also by other 8-bit characters (whose encoding is likely",
+  "to be part of the ISO-8859 family). Note that each time this command",
+  "modifies the buffer encoding, it also resets the undo buffer.",
 };
 
 const char * const UTF8AUTO_HELP[14] = {
@@ -1722,7 +1843,7 @@ const char * const UTF8IO_HELP[13] = {
   "respectively. An `@' will appear on the status bar if the flag is true.",
 };
 
-const char * const VERBOSEMACROS_HELP[17] = {
+const char * const VERBOSEMACROS_HELP[21] = {
   "Syntax: VerboseMacros [0|1]",
   "Abbreviation: VM",
   "",
@@ -1739,6 +1860,10 @@ const char * const VERBOSEMACROS_HELP[17] = {
   "   The only reason to use this flag is when recording a macro that will",
   "be played a great number of times. Automatic preferences files are too",
   "short to be an issue with respect to execution timing.",
+  "",
+  "   The `VerboseMacros' setting is saved in your `~/.ne/.default#ap' file",
+  "when you use the `SaveDefPrefs' command or the `Save Def Prefs' menu.",
+  "It is not saved by the `SaveAutoPrefs' command.",
 };
 
 const char * const VISUALBELL_HELP[10] = {
