@@ -1,7 +1,7 @@
 /* Help string externs.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2005 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -22,8 +22,9 @@
 
 
 extern const char * const ABOUT_HELP[ 5 ];
-extern const char * const ADJUSTVIEW_HELP[ 20 ];
+extern const char * const ADJUSTVIEW_HELP[ 26 ];
 extern const char * const ALERT_HELP[ 5 ];
+extern const char * const AUTOCOMPLETE_HELP[ 20 ];
 extern const char * const AUTOINDENT_HELP[ 22 ];
 extern const char * const AUTOPREFS_HELP[ 14 ];
 extern const char * const BACKSPACE_HELP[ 6 ];
@@ -41,16 +42,18 @@ extern const char * const CUT_HELP[ 5 ];
 extern const char * const DELETECHAR_HELP[ 10 ];
 extern const char * const DELETEEOL_HELP[ 10 ];
 extern const char * const DELETELINE_HELP[ 9 ];
+extern const char * const DELETENEXTWORD_HELP[ 6 ];
+extern const char * const DELETEPREVWORD_HELP[ 8 ];
 extern const char * const DOUNDO_HELP[ 20 ];
 extern const char * const ERASE_HELP[ 6 ];
 extern const char * const ESCAPE_HELP[ 7 ];
 extern const char * const ESCAPETIME_HELP[ 16 ];
 extern const char * const EXEC_HELP[ 13 ];
 extern const char * const EXIT_HELP[ 6 ];
-extern const char * const FASTGUI_HELP[ 15 ];
+extern const char * const FASTGUI_HELP[ 19 ];
 extern const char * const FIND_HELP[ 12 ];
 extern const char * const FINDREGEXP_HELP[ 12 ];
-extern const char * const FLAGS_HELP[ 27 ];
+extern const char * const FLAGS_HELP[ 30 ];
 extern const char * const FLASH_HELP[ 6 ];
 extern const char * const FREEFORM_HELP[ 18 ];
 extern const char * const GOTOBOOKMARK_HELP[ 13 ];
@@ -60,9 +63,11 @@ extern const char * const GOTOMARK_HELP[ 9 ];
 extern const char * const HELP_HELP[ 15 ];
 extern const char * const HEXCODE_HELP[ 7 ];
 extern const char * const INSERT_HELP[ 12 ];
-extern const char * const INSERTCHAR_HELP[ 20 ];
+extern const char * const INSERTCHAR_HELP[ 21 ];
 extern const char * const INSERTLINE_HELP[ 7 ];
 extern const char * const INSERTSTRING_HELP[ 12 ];
+extern const char * const INSERTTAB_HELP[ 7 ];
+extern const char * const KEYCODE_HELP[ 8 ];
 extern const char * const LINEDOWN_HELP[ 6 ];
 extern const char * const LINEUP_HELP[ 6 ];
 extern const char * const LOADAUTOPREFS_HELP[ 7 ];
@@ -114,6 +119,7 @@ extern const char * const REPEATLAST_HELP[ 26 ];
 extern const char * const REPLACE_HELP[ 46 ];
 extern const char * const REPLACEALL_HELP[ 12 ];
 extern const char * const REPLACEONCE_HELP[ 10 ];
+extern const char * const REQUESTORDER_HELP[ 15 ];
 extern const char * const RIGHTMARGIN_HELP[ 12 ];
 extern const char * const SAVE_HELP[ 13 ];
 extern const char * const SAVEAS_HELP[ 14 ];
@@ -125,10 +131,11 @@ extern const char * const SAVEPREFS_HELP[ 11 ];
 extern const char * const SEARCHBACK_HELP[ 14 ];
 extern const char * const SELECTDOC_HELP[ 14 ];
 extern const char * const SETBOOKMARK_HELP[ 10 ];
-extern const char * const STATUSBAR_HELP[ 19 ];
+extern const char * const STATUSBAR_HELP[ 23 ];
 extern const char * const SUSPEND_HELP[ 6 ];
-extern const char * const SYNTAX_HELP[ 41 ];
+extern const char * const SYNTAX_HELP[ 42 ];
 extern const char * const SYSTEM_HELP[ 10 ];
+extern const char * const TABS_HELP[ 24 ];
 extern const char * const TABSIZE_HELP[ 9 ];
 extern const char * const THROUGH_HELP[ 15 ];
 extern const char * const TOGGLESEOF_HELP[ 10 ];
@@ -143,7 +150,7 @@ extern const char * const UNSETBOOKMARK_HELP[ 10 ];
 extern const char * const UTF8_HELP[ 20 ];
 extern const char * const UTF8AUTO_HELP[ 14 ];
 extern const char * const UTF8IO_HELP[ 13 ];
-extern const char * const VERBOSEMACROS_HELP[ 17 ];
+extern const char * const VERBOSEMACROS_HELP[ 21 ];
 extern const char * const VISUALBELL_HELP[ 10 ];
 extern const char * const WORDWRAP_HELP[ 12 ];
 
