@@ -1,7 +1,7 @@
 /* terminfo emulation through GNU termcap code.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -251,8 +251,8 @@ int setupterm(const char * const dummy1, const int dunmmy2, const int * const du
 
 	ne_generic_type = tgetflag("gn");
 
-	if ((ne_lines = tgetnum("li")) < 0) ne_lines = 25;
-	if ((ne_columns = tgetnum("co")) < 0) ne_columns = 80;
+	if ((ne_lines = tgetnum("li")) <= 0) ne_lines = 25;
+	if ((ne_columns = tgetnum("co")) <= 0) ne_columns = 80;
 
 	l = c = 0;
 	if (s = getenv("LINES")) l = atoi(s);
