@@ -13,7 +13,7 @@ $| = 1;
 
 my $copyright =
   qq[	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2005 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -115,7 +115,7 @@ foreach ( @infofiles )
 # all the hashed commands and command abbreviations, without collisions.
 #
 # The tables are precompiled into the file hash.c. The file hash.h
-# contains instead a #define which specify the dimension of the hash
+# contains instead #defines which specify the dimensions of the hash
 # tables.
 #
 # The hash function we choose is very simple: it considers a string of
@@ -379,7 +379,7 @@ sub tail_names_c
 /* These are extras that are very useful in the default menus and key bindings. */
 
 const char PLAYONCE_ABBREV[] = "PL 1";
-const char CENTERVIEW_ABBREV[] = "AV C";
+const char MIDDLEVIEW_ABBREV[] = "AV M";
 
 /* This is the NULL-terminated, ordered list of names, useful for help etc. */
 
@@ -424,7 +424,7 @@ sub tail_names_h
 
 /* These are extras that are very useful in the default menus and key bindings. */
 extern const char PLAYONCE_ABBREV[];
-extern const char CENTERVIEW_ABBREV[];
+extern const char MIDDLEVIEW_ABBREV[];
 
 /* This is the NULL-terminated, ordered list of names, useful for help etc. */
 
@@ -570,6 +570,11 @@ sub body_hash_h
   
 sub tail_hash_h
   {
+    my $max_command_width = 0;
+    foreach my $cmd ( keys %commands )
+      {
+        $max_command_width = length $cmd if length $cmd > $max_command_width;
+      }
     print HASH_H <<"_EOT_";
 /* These vectors are hash tables with no conflicts. For each command, the
 element indexed by the hashed name of the command contains the command
@@ -583,6 +588,14 @@ so they can be moved to the text segment. */
 extern const unsigned char hash_table[HASH_TABLE_SIZE];
 extern const unsigned char short_hash_table[HASH_TABLE_SIZE];
 
+/* The maximum width for a command is used when displaying the command
+names with the string requester. For example, 18 would allow four columns
+on an 80x25 screen.
+*/
+
+#define MAX_COMMAND_WIDTH		$max_command_width
+
+
 /* This file was automatically generated by $0. */
 _EOT_
   }
