@@ -1,7 +1,7 @@
 /* Input line handling.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -93,7 +93,7 @@ char request_char(const buffer * const b, const char * const prompt, const char
 	set_attr(0);
 	print_prompt(prompt);
 
-	if (default_value) output_char(default_value, -1, FALSE);
+	if (default_value) output_char(default_value, 0, FALSE);
 	move_cursor(b->cur_y, b->cur_x);
 
 	while(TRUE) {
@@ -210,8 +210,8 @@ static void add_to_history(const unsigned char * const str) {
 
 	move_to_bof(history_buff);
 
-	/* This insert_stream() takes care of adding a line, including a life
-		feed at the end. */
+	/* This insert_stream() takes care of adding a line, including a line-feed
+	   at the end. */
 
 	insert_stream(history_buff,
 						history_buff->cur_line_desc,
@@ -268,7 +268,7 @@ static void input_refresh(void) {
 	move_cursor(ne_lines - 1, start_x);
 	for(i = start_x, j = offset; j < len; i += get_char_width(&input_buffer[j], encoding), j = next_pos(input_buffer, j, encoding)) {
 		if (i + get_char_width(&input_buffer[j], encoding) >= ne_columns) break;
-		output_char(get_char(&input_buffer[j], encoding), -1, encoding);
+		output_char(get_char(&input_buffer[j], encoding), 0, encoding);
 	}
 	clear_to_eol();
 	fflush(stdout);
@@ -330,12 +330,12 @@ static void input_move_right(const int do_refresh) {
 				move_cursor(ne_lines - 1, start_x);
 				delete_chars(width);
 				move_cursor(ne_lines - 1, x - x_delta);
-				output_char(get_char(&input_buffer[prev_pos], encoding), -1, encoding);
+				output_char(get_char(&input_buffer[prev_pos], encoding), 0, encoding);
 
 				i = x;
 				j = pos;
 				while(j < len && i + (width = get_char_width(&input_buffer[j], encoding)) < ne_columns) {
-					output_char(get_char(&input_buffer[j], encoding), -1, encoding);
+					output_char(get_char(&input_buffer[j], encoding), 0, encoding);
 					i += width;
 					j = next_pos(input_buffer, j, encoding);
 				}
@@ -390,7 +390,7 @@ static void input_next_word(void) {
 
 	while(pos < len) {
 		const int c = get_char(&input_buffer[pos], encoding);
-		if (ne_isspace(c, encoding) || ne_ispunct(c, encoding)) space_skipped = TRUE;
+		if (!ne_isword(c, encoding)) space_skipped = TRUE;
 		else if (space_skipped) break;
 		input_move_right(FALSE);
 	}
@@ -408,7 +408,7 @@ static void input_prev_word(void) {
 	while(pos > 0) {
 		input_move_left(FALSE);
 		c = get_char(&input_buffer[pos], encoding);
-		if (!(ne_isspace(c, encoding) || ne_ispunct(c, encoding))) word_skipped = TRUE;
+		if (ne_isword(c, encoding)) word_skipped = TRUE;
 		else if (word_skipped) {
 			input_move_right(FALSE);
 			break;
@@ -505,8 +505,8 @@ char *request(const char *prompt, const char * const default_string, const int a
 				move_cursor(ne_lines - 1, x);
 
 				if (x < ne_columns - c_width) {
-					if (pos == len - c_len) output_char(c, -1, encoding);
-					else if (char_ins_del_ok) insert_char(c, -1, encoding);
+					if (pos == len - c_len) output_char(c, 0, encoding);
+					else if (char_ins_del_ok) insert_char(c, 0, encoding);
 					else input_refresh();
 				}
 
@@ -658,7 +658,7 @@ char *request(const char *prompt, const char * const default_string, const int a
 							if (j < len) {
 								move_cursor(ne_lines - 1, i);
 								while(j < len && i + get_char_width(&input_buffer[j], encoding) < ne_columns) {
-									output_char(get_char(&input_buffer[j], encoding), -1, encoding);
+									output_char(get_char(&input_buffer[j], encoding), 0, encoding);
 									i += get_char_width(&input_buffer[j], encoding);
 									j = next_pos(input_buffer, j, encoding);
 								}
