@@ -1,7 +1,7 @@
 /* Terminfo database scanning and keyboard escape sequence matching functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -27,10 +27,6 @@
 #include <errno.h>
 #include <string.h>
 
-#ifdef _AMIGA
-#include <proto/dos.h>
-#endif
-
 
 /* Maximum number of key definitions from terminfo */
 
@@ -276,7 +272,7 @@ void read_key_capabilities(void) {
 
 	assert(num_keys < MAX_TERM_KEY - 1);
 
-	D(fprintf(stderr,"Got %d keys from terminfo\n\r", num_keys);)
+	D(fprintf(stderr,"Got %d keys from terminfo\n", num_keys);)
 
 	qsort(key, num_keys, sizeof(term_key), keycmp);
 
@@ -284,7 +280,7 @@ void read_key_capabilities(void) {
 
 		Unfortunately, quite a few terminfo and termcap entries out there have
 		bad values for cursor key capability strings. (The f# values are
-		generally is sad shape too, but that's a much larger problem.)  However,
+		generally in sad shape too, but that's a much larger problem.)  However,
 		certain escape sequences are quite common among large sets of terminals,
 		and so we define the most common ones here.
 		
@@ -405,8 +401,6 @@ void set_escape_time(const int new_escape_time) {
 	timeout value (in tenth of a second) is positive, VMIN is set to 0,
 	otherwise to 1. */
 
-#ifndef _AMIGA
-
 static void set_termios_timeout(const int timeout) {
 	struct termios termios;
 
@@ -418,8 +412,6 @@ static void set_termios_timeout(const int timeout) {
 	tcsetattr(0, TCSANOW, &termios);
 }
 
-#endif
-
 
 /* Reads in characters, and tries to match them with the sequences
 	corresponding to special keys. Returns a positive number, denoting
@@ -524,25 +516,18 @@ int get_key_code(void) {
 		
 		fflush(stdout);
 
-#ifdef _AMIGA
-		if (!partial_match || WaitForChar(Input(), 100000 * escape_time)) c = getchar();
-		else c = EOF;
-#else
 		if (partial_match) set_termios_timeout(escape_time);
 
 		errno = 0;
 		c = getchar();
-
 		e = errno;
-		/* This is necessary to circumvent the slightly different behaviour of getc() in Linux and BSD. */
-		clearerr(stdin);
-		if (c == EOF && (!partial_match || e) && e != EINTR) kill(getpid(), SIGTERM);
-
-		if (c == EOF && e == EINTR) return -NE_KEY_IGNORE - 1;
-
+		
 		if (partial_match) set_termios_timeout(0);
 
-#endif
+		/* This is necessary to circumvent the slightly different behaviour of getc() in Linux and BSD. */
+		clearerr(stdin);
+
+		if (c == EOF && (!partial_match || e) && e != EINTR) kill(getpid(), SIGTERM);
 
 		partial_match = FALSE;
 
@@ -553,9 +538,8 @@ int get_key_code(void) {
 			if (cur_len) {
 
 				/* We ran out of time. If our match was not UTF-8, then the current
-					partial match is discarded, and the first character of the
-					keyboard buffer is returned. Otherwise, we reject the partially
-					received UTF-8 sequence. */
+					we return the first character of the keyboard buffer is returned. 
+					Otherwise, we discard the partially received UTF-8 sequence. */
 
 				if (partial_is_utf8) cur_len = last_match = partial_is_utf8 = 0;
 				else {
@@ -567,4 +551,3 @@ int get_key_code(void) {
 		}
 	}
 }
-
