@@ -1,7 +1,7 @@
 /* Menu handling function. Includes also key and menu configuration parsing.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1993-1998 Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -41,6 +41,10 @@
 
 #define MAX_FLAG_STRING_SIZE 32
 
+/* The maximum length of a message. */
+
+#define MAX_MESSAGE_LENGTH 1024
+
 /* The name of the menu configuration file. */
 
 #define MENU_CONF_NAME ".menus"
@@ -80,28 +84,28 @@ typedef struct {
 	const menu_item *items;
 } menu;
 
-#ifndef SHELLKEYS
-  #define PICK(A,B,C,D) {A,B},
+#ifndef ALTPAGING
+	#define PICK(A,B,C,D) {A,B},
 #else
-  #define PICK(A,B,C,D) {C,D},
+	#define PICK(A,B,C,D) {C,D},
 #endif
 
 /* The following structures describe ne's standard menus. */
 
 static menu_item file_item[] =
 	{
-		{ "Open...   ^O", OPEN_ABBREV },
-		{ "Open New... ", OPENNEW_ABBREV },
-		{ "Save      ^S", SAVE_ABBREV },
-		{ "Save As...  ", SAVEAS_ABBREV },
-		{ "Quit Now  [Q", QUIT_ABBREV },
-		{ "Save&Exit [X", EXIT_ABBREV },
-		{ "About       ", ABOUT_ABBREV }
+		{ "Open...     ^O", OPEN_ABBREV },
+		{ "Open New... [N", OPENNEW_ABBREV },
+		{ "Save        ^S", SAVE_ABBREV },
+		{ "Save As...    ", SAVEAS_ABBREV },
+		{ "Quit Now    [Q", QUIT_ABBREV },
+		{ "Save&Exit   [X", EXIT_ABBREV },
+		{ "About         ", ABOUT_ABBREV }
 	};
 
 static const menu_item documents_item[] =
 	{
-      { "New       ^D", NEWDOC_ABBREV },
+		{ "New       ^D", NEWDOC_ABBREV },
 		{ "Clear       ", CLEAR_ABBREV },
 		{ "Close     ^Q", CLOSEDOC_ABBREV },
 		{ "Next   f2/[D", NEXTDOC_ABBREV },
@@ -115,12 +119,15 @@ static const menu_item edit_item[] =
 		{ "Cut         ^X", CUT_ABBREV },
 		{ "Copy        ^C", COPY_ABBREV },
 		{ "Paste       ^V", PASTE_ABBREV },
-		{ "Erase         ", ERASE_ABBREV },
-		{ "Through     [T", THROUGH_ABBREV },
-		{ "Delete Line ^Y", DELETELINE_ABBREV },
-		{ "Delete EOL  [Y", DELETEEOL_ABBREV },
 		{ "Mark Vert   ^@", MARKVERT_ABBREV },
 		{ "Paste Vert  ^W", PASTEVERT_ABBREV },
+		{ "Through     [T", THROUGH_ABBREV },
+		{ "Erase         ", ERASE_ABBREV },
+		{ "Delete EOL  [Y", DELETEEOL_ABBREV },
+		{ "Delete Line ^Y", DELETELINE_ABBREV },
+		{ "Undel Line  ^U", UNDELLINE_ABBREV },
+		{ "Del Prev Word ", DELETEPREVWORD_ABBREV },
+		{ "Del Next Word ", DELETENEXTWORD_ABBREV },
 		{ "Open Clip   [O", OPENCLIP_ABBREV },
 		{ "Save Clip   [S", SAVECLIP_ABBREV }
 	};
@@ -156,21 +163,21 @@ static const menu_item macros_item[] =
 
 static const menu_item extras_item[] =
 	{
-		{ "Exec...     ^K", EXEC_ABBREV },
-		{ "Suspend     ^Z", SUSPEND_ABBREV },
-		{ "Help...    f10", HELP_ABBREV },                     
-		{ "Refresh     ^L", REFRESH_ABBREV },
-		{ "Undo     f5/[U", UNDO_ABBREV },
-		{ "Redo     f6/[R", REDO_ABBREV },
-		{ "Undel Line  ^U", UNDELLINE_ABBREV },
-		{ "Center        ", CENTER_ABBREV },
-		{ "Paragraph   [P", PARAGRAPH_ABBREV },
-		{ "Adjust View ^^", ADJUSTVIEW_ABBREV },
-		{ "Center View [C", CENTERVIEW_ABBREV },
-		{ "ToUpper     [V", TOUPPER_ABBREV },
-		{ "ToLower     [L", TOLOWER_ABBREV },
-		{ "Capitalize    ", CAPITALIZE_ABBREV },
-		{ "UTF-8         ", UTF8_ABBREV }
+		{ "Exec...      ^K", EXEC_ABBREV },
+		{ "Suspend      ^Z", SUSPEND_ABBREV },
+		{ "Help...     f10", HELP_ABBREV },
+		{ "Refresh      ^L", REFRESH_ABBREV },
+		{ "Undo      f5/[U", UNDO_ABBREV },
+		{ "Redo      f6/[R", REDO_ABBREV },
+		{ "Center         ", CENTER_ABBREV },
+		{ "Paragraph    [P", PARAGRAPH_ABBREV },
+		{ "Adjust View  ^^", ADJUSTVIEW_ABBREV },
+		{ "Middle View  [C", MIDDLEVIEW_ABBREV },
+		{ "ToUpper      [V", TOUPPER_ABBREV },
+		{ "ToLower      [L", TOLOWER_ABBREV },
+		{ "Capitalize     ", CAPITALIZE_ABBREV },
+		{ "AutoComplete [I", AUTOCOMPLETE_ABBREV },
+		{ "UTF-8          ", UTF8_ABBREV }
 	};
 
 
@@ -181,10 +188,10 @@ static const menu_item navigation_item[] =
 		{ "Move Right      ", MOVERIGHT_ABBREV },
 		{ "Line Up         ", LINEUP_ABBREV },
 		{ "Line Down       ", LINEDOWN_ABBREV },
-PICK(   "Prev Page     ^P", PREVPAGE_ABBREV    , "Page Up       ^P", PAGEUP_ABBREV)
-PICK(   "Next Page     ^N", NEXTPAGE_ABBREV    , "Page Down     ^N", PAGEDOWN_ABBREV)
-PICK(   "Page Up         ", PAGEUP_ABBREV      , "Prev Page       ", PREVPAGE_ABBREV)
-PICK(   "Page Down       ", PAGEDOWN_ABBREV    , "Next Page       ", NEXTPAGE_ABBREV)
+PICK(   "Prev Page     ^P", PREVPAGE_ABBREV    , "Prev Page       ", PREVPAGE_ABBREV)
+PICK(   "Next Page     ^N", NEXTPAGE_ABBREV    , "Next Page       ", NEXTPAGE_ABBREV)
+PICK(   "Page Up         ", PAGEUP_ABBREV      , "Page Up       ^P", PAGEUP_ABBREV)
+PICK(   "Page Down       ", PAGEDOWN_ABBREV    , "Page Down     ^N", PAGEDOWN_ABBREV)
 		{ "Start Of File [A", MOVESOF_ABBREV },
 		{ "End Of File   [E", MOVEEOF_ABBREV },
 		{ "Start Of Line ^A", MOVESOL_ABBREV },
@@ -202,6 +209,7 @@ PICK(   "Page Down       ", PAGEDOWN_ABBREV    , "Next Page       ", NEXTPAGE_AB
 static const menu_item prefs_item[] =
 	{
 		{ "Tab Size...     ", TABSIZE_ABBREV },
+		{ "Tabs/Spaces     ", TABS_ABBREV },
 		{ "Insert/Over  Ins", INSERT_ABBREV },
 		{ "Free Form       ", FREEFORM_ABBREV },
 		{ "Status Bar      ", STATUSBAR_ABBREV },
@@ -209,7 +217,8 @@ static const menu_item prefs_item[] =
 		{ "Fast GUI        ", FASTGUI_ABBREV },
 		{ "Word Wrap     [W", WORDWRAP_ABBREV },
 		{ "Right Margin    ", RIGHTMARGIN_ABBREV },
-		{ "Auto Indent   [I", AUTOINDENT_ABBREV },
+		{ "Auto Indent     ", AUTOINDENT_ABBREV },
+		{ "Request Order   ", REQUESTORDER_ABBREV },
 		{ "Preserve CR     ", PRESERVECR_ABBREV },
 		{ "Save CR/LF    [Z", CRLF_ABBREV },
 		{ "Load Prefs...   ", LOADPREFS_ABBREV },
@@ -223,7 +232,7 @@ static menu def_menus[DEF_MENU_NUM] = {
 	{
 		sizeof(file_item) / sizeof(menu_item),
 		1,
-		12,
+		14,
 		0,
 		"File",
 		file_item
@@ -263,7 +272,7 @@ static menu def_menus[DEF_MENU_NUM] = {
 	{
 		sizeof(extras_item) / sizeof(menu_item),
 		35,
-		14,
+		15,
 		0,
 		"Extras",
 		extras_item
@@ -299,46 +308,40 @@ static int current_menu = 0, menu_num = DEF_MENU_NUM;
 static menu *menus = def_menus;
 
 
-#ifdef DONT_COMPILE_THIS_PART
-int dump_config(void)
-  {
-    int menu, item, key;
-    FILE *f;
-    
-    if (!(f = fopen("dump_config","w")) ) return ERROR;
-    
-    for (menu = 0; menu < menu_num; menu++)
-      {
-        fprintf(f,"%s      \"%s\"\n", MENU_KEYWORD, menus[menu].text );
-        for (item = 0; item < menus[menu].item_num; item++)
-          {
-            fprintf(f,"     %s \"%s\" \"%s\"\n", ITEM_KEYWORD,
-                                                menus[menu].items[item].text,
-                                                menus[menu].items[item].command_line);
-          }
-        fprintf(f,"\n");
-      }
-    
-    for (key = 0; key < NUM_KEYS; key++)
-      {
-        if (key_binding[key] && key_binding[key][0])
-          {
-            fprintf(f,"%s %4x %s\n",KEY_KEYWORD, key, key_binding[key] );
-          }
-      }
-    fclose(f);
-    return OK;
-  }
+#ifdef NE_TEST
+int dump_config(void) {
+	int menu, item, key;
+	FILE *f;
+
+	if (!(f = fopen("ne_test_dump_config","w")) ) return ERROR;
+
+	for (menu = 0; menu < menu_num; menu++) {
+		fprintf(f,"%s \"%s\"\n", MENU_KEYWORD, menus[menu].text );
+		for (item = 0; item < menus[menu].item_num; item++) {
+			fprintf(f,"%s \"%s\" \"%s\"\n", ITEM_KEYWORD,
+			                                    menus[menu].items[item].text,
+			                                    menus[menu].items[item].command_line);
+		}
+		fprintf(f,"\n");
+	}
+
+	for (key = 0; key < NUM_KEYS; key++) {
+		if (key_binding[key] && key_binding[key][0])
+			fprintf(f,"%s\t%4x\t%s\n",KEY_KEYWORD, key, key_binding[key] );
+	}
+	fclose(f);
+	return OK;
+}
 #endif
-  
+
 static void draw_cur_item(const int n) {
 	move_cursor(menus[n].cur_item + 1, menus[n].xpos);
-	if (!cur_buffer->opt.fast_gui && standout_ok) output_chars(menus[n].items[menus[n].cur_item].text, NULL, menus[n].width - (cursor_on_off_ok ? 0 : 1), TRUE);
+	if (!fast_gui && standout_ok) output_chars(menus[n].items[menus[n].cur_item].text, NULL, menus[n].width - (cursor_on_off_ok ? 0 : 1), TRUE);
 }
 
 
 static void undraw_cur_item(const int n) {
-	if (!cur_buffer->opt.fast_gui && standout_ok)  {
+	if (!fast_gui && standout_ok)  {
 		set_attr(0);
 		standout_on();
 		move_cursor(1 + menus[n].cur_item, menus[n].xpos);
@@ -502,7 +505,7 @@ static void draw_first_menu(void) {
 
 	set_attr(0);
 	standout_on();
-	if (!cur_buffer->opt.fast_gui && standout_ok) cursor_off();
+	if (!fast_gui && standout_ok) cursor_off();
 
 	while(i < ne_columns) {
 		output_string(" ", FALSE);
@@ -522,7 +525,7 @@ static void draw_first_menu(void) {
 
 static void undraw_last_menu(void) {
 	undraw_menu(current_menu);
-	update_line(cur_buffer, 0, FALSE);
+	update_line(cur_buffer, 0, FALSE, FALSE);
 	cursor_on();
 }
 
@@ -574,9 +577,10 @@ char *gen_flag_string(const buffer * const b) {
 	string[i++] = b->opt.word_wrap      ? 'w' : '-';
 	string[i++] = b->opt.free_form      ? 'f' : '-';
 	string[i++] = b->opt.auto_prefs     ? 'p' : '-';
-	string[i++] = b->opt.verbose_macros ? 'v' : '-';
+	string[i++] = verbose_macros        ? 'v' : '-';
 	string[i++] = b->opt.do_undo        ? 'u' : '-';
 	string[i++] = b->opt.read_only      ? 'r' : '-';
+	string[i++] = b->opt.tabs           ? 't' : '-';
 	string[i++] = b->opt.binary         ? 'B' : '-';
 	string[i++] = b->marking            ? (b->mark_is_vertical ? 'V' :'M') : '-';
 	string[i++] = b->recording          ? 'R' : '-';
@@ -586,7 +590,7 @@ char *gen_flag_string(const buffer * const b) {
 	string[i++] = io_utf8               ? '@' : '-';
 	string[i++] = b->encoding != ENC_8_BIT? (b->encoding == ENC_UTF8 ? 'U' : 'A') : '8';
 
-	if (b->opt.hex_code && !b->opt.fast_gui) {
+	if (b->opt.hex_code && !fast_gui) {
 		string[i++] = ' ';
 		if (ch > 0xFFFF) {
 			string[i++] = "0123456789abcdef"[(ch >> 28) & 0x0f];
@@ -631,7 +635,6 @@ void draw_status_bar(void) {
 
 	char *p;
 	int len;
-	int i;
 
 	if (showing_msg) {
 		showing_msg = FALSE;
@@ -641,53 +644,71 @@ void draw_status_bar(void) {
 
 	set_attr(0);
 
-	if (!bar_gone && cur_buffer->opt.status_bar) {
+	if (!bar_gone && status_bar) {
+		const int new_percent = (int)floor(((cur_buffer->cur_line + 1) * 100.0) / cur_buffer->num_lines);
 		/* This is the space occupied up to "L:", included. */
-		const int offset = cur_buffer->opt.fast_gui || !standout_ok ? 5: 3;
-		const int update_linecols = y != cur_buffer->cur_line || 
-		                            x != cur_buffer->win_x + cur_buffer->cur_x ||
-		                            percent != ((cur_buffer->cur_line + 1) * 100) / cur_buffer->num_lines;
+		const int offset = fast_gui || !standout_ok ? 5: 3;
+		const int update_x = x != cur_buffer->win_x + cur_buffer->cur_x;
+		const int update_y = y != cur_buffer->cur_line;
+		const int update_percent = percent != new_percent;
+		const int update_flags = strcmp(flag_string, p = gen_flag_string(cur_buffer));
+		const int update = update_x || update_y || update_percent || update_flags;
 
-		if (!cur_buffer->opt.fast_gui && standout_ok) standout_on();
+		if (!update) return;
+
+		if (!fast_gui && standout_ok) standout_on();
 		
 		x = cur_buffer->win_x + cur_buffer->cur_x;
 		y = cur_buffer->cur_line;
-		percent = ((cur_buffer->cur_line + 1) * 100) / cur_buffer->num_lines;
-		i = sprintf(bar_buffer, "%8d C:%8d %3d", y + 1, x + 1, percent);
+		percent = new_percent;
 
-		if (update_linecols) {
+		if (update_y) {
 			move_cursor(ne_lines - 1, offset);
-			output_chars(bar_buffer, NULL, i, TRUE);
+			len = sprintf(bar_buffer, "%9d", y + 1);
+			output_chars(bar_buffer, NULL, len, TRUE);
 		}
 
-		if (strcmp(flag_string, p = gen_flag_string(cur_buffer))) {
+		if (update_x) {
+			move_cursor(ne_lines - 1, offset + 12);
+			len = sprintf(bar_buffer, "%9d", x + 1);
+			output_chars(bar_buffer, NULL, len, TRUE);
+		}
+
+		if (update_percent) {
+			move_cursor(ne_lines - 1, offset + 22);
+			len = sprintf(bar_buffer, "%3d", percent);
+			output_chars(bar_buffer, NULL, len, TRUE);
+		}
+
+		if (update_flags) {
 			strcpy(flag_string, p);
-			move_cursor(ne_lines - 1, i + offset + 2);
+			move_cursor(ne_lines - 1, offset + 27);
 			output_string(flag_string, TRUE);
 		}
 
-		if (!cur_buffer->opt.fast_gui && standout_ok) standout_off();
+		if (!fast_gui && standout_ok) standout_off();
 		return;
 	}
 
 
-	if (cur_buffer->opt.status_bar) {
+	if (status_bar) {
+		percent = (int)floor(((cur_buffer->cur_line + 1) * 100.0) / cur_buffer->num_lines);
 		move_cursor(ne_lines - 1, 0);
-		if (!cur_buffer->opt.fast_gui && standout_ok) standout_on();
+		if (!fast_gui && standout_ok) standout_on();
 
 		strcpy(flag_string, gen_flag_string(cur_buffer));
 
 		x = cur_buffer->win_x + cur_buffer->cur_x;
 		y = cur_buffer->cur_line;
 
-		len = sprintf(bar_buffer, cur_buffer->opt.fast_gui || !standout_ok ? ">> L:%8d C:%8d %3d%% %s " : " L:%8d C:%8d %3d%% %s ", y + 1, x + 1, ((y + 1) * 100) / cur_buffer->num_lines, flag_string);
+		len = sprintf(bar_buffer, fast_gui || !standout_ok ? ">> L:%9d C:%9d %3d%% %s " : " L:%9d C:%9d %3d%% %s ", y + 1, x + 1, percent, flag_string);
 
 		move_cursor(ne_lines - 1, 0);
 		output_chars(bar_buffer, NULL, len, TRUE);
 
 		if (len < ne_columns - 1) {
 			if (cur_buffer->filename) {
-	 			/* This is a bit complicated because we have to compute the width of the filename first, and then 
+	 			/* This is a bit complicated because we have to compute the width of the filename first, and then
 				discard initial characters until the remaning part will fit. */
 
 				const int encoding = detect_encoding(cur_buffer->filename, strlen(cur_buffer->filename));
@@ -703,7 +724,7 @@ void draw_status_bar(void) {
 			else output_string(UNNAMED_NAME, FALSE);
 		}
 
-		if (!cur_buffer->opt.fast_gui && standout_ok) {
+		if (!fast_gui && standout_ok) {
 			output_spaces(ne_columns, NULL);
 			standout_off();
 		}
@@ -720,28 +741,39 @@ void draw_status_bar(void) {
 
 
 /* Prints a message over the status bar. It also sets showing_msg and
-	bar_gone. */
+	bar_gone. If message is NULL and showing_msg is true, we reprint
+	the last message. That necessitates caching the message when it
+	isn't NULL. */
 
 void print_message(const char * const message) {
 
-	move_cursor(ne_lines - 1, 0);
-
-	set_attr(0);
-
-	if (cur_buffer->opt.fast_gui || !standout_ok || !cur_buffer->opt.status_bar) {
-		clear_to_eol();
-		output_string(message, TRUE);
+	static char msg_cache[MAX_MESSAGE_LENGTH];
+	
+	if (message) {
+		strncpy(msg_cache, message, MAX_MESSAGE_LENGTH);
+		msg_cache[MAX_MESSAGE_LENGTH - 1] = '\0';
 	}
-	else {
-		standout_on();
-		output_string(message, TRUE);
-		output_spaces(ne_columns - strlen(message), NULL);
-		standout_off();
+	
+   if (message || showing_msg) {
+		move_cursor(ne_lines - 1, 0);
+
+		set_attr(0);
+
+		if (fast_gui || !standout_ok || !status_bar) {
+			clear_to_eol();
+			output_string(msg_cache, TRUE);
+		}
+		else {
+			standout_on();
+			output_string(msg_cache, TRUE);
+			output_spaces(ne_columns - strlen(msg_cache), NULL);
+			standout_off();
+		}
+
+		fflush(stdout);
+
+		showing_msg = TRUE;
 	}
-
-	fflush(stdout);
-
-	showing_msg = TRUE;
 }
 
 
@@ -876,7 +908,7 @@ static void error_in_menu_configuration(const int line, const char * const s) {
 }
 
 
-void get_menu_configuration(const char * menu_conf_name) {
+static void get_menu_conf(const char * menu_conf_name, char * (exists_prefs_func)()) {
 
 	char  *prefs_dir, *menu_conf;
 	char_stream *cs;
@@ -887,7 +919,7 @@ void get_menu_configuration(const char * menu_conf_name) {
 
 	if (!menu_conf_name) menu_conf_name = MENU_CONF_NAME;
 
-	if (prefs_dir = exists_prefs_dir()) {
+	if (prefs_dir = exists_prefs_func()) {
 		if (menu_conf = malloc(strlen(prefs_dir) + strlen(menu_conf_name) + 1)) {
 			strcat(strcpy(menu_conf, prefs_dir), menu_conf_name);
 
@@ -984,6 +1016,13 @@ void get_menu_configuration(const char * menu_conf_name) {
 	}
 }
 
+/* Menu configs are all or nothing, so if the user has one,
+   skip any global one. */
+void get_menu_configuration(const char * menu_conf_name) {
+	get_menu_conf(menu_conf_name, exists_prefs_dir);
+	if (menus == def_menus) get_menu_conf(menu_conf_name, exists_gprefs_dir);
+}
+
 
 static void error_in_key_bindings(const int line, const char * const s) {
 
@@ -991,7 +1030,7 @@ static void error_in_key_bindings(const int line, const char * const s) {
 	exit(0);
 }
 
-void get_key_bindings(const char * key_bindings_name) {
+static void get_key_bind(const char * key_bindings_name, char * (exists_prefs_func)()) {
 	char *prefs_dir, *key_bindings;
 	char_stream *cs;
 	unsigned char *p;
@@ -999,7 +1038,7 @@ void get_key_bindings(const char * key_bindings_name) {
 
 	if (!key_bindings_name) key_bindings_name = KEY_BINDINGS_NAME;
 
-	if (prefs_dir = exists_prefs_dir()) {
+	if (prefs_dir = exists_prefs_func()) {
 		if (key_bindings = malloc(strlen(prefs_dir) + strlen(key_bindings_name) + 1)) {
 			strcat(strcpy(key_bindings, prefs_dir), key_bindings_name);
 
@@ -1037,3 +1076,11 @@ void get_key_bindings(const char * key_bindings_name) {
 		}
 	}
 }
+
+/* Key bindings override easily, so pull in any global bindings
+   first, then override with the users bindings. */
+void get_key_bindings(const char * key_bindings_name) {
+	get_key_bind(key_bindings_name, exists_gprefs_dir);
+	get_key_bind(key_bindings_name, exists_prefs_dir);
+}
+
