@@ -1,7 +1,7 @@
 /* Names and abbreviations of all the commands.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2005 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -36,6 +36,9 @@ const char ADJUSTVIEW_ABBREV[] = "AV";
 const char ALERT_NAME[]   = "Alert";
 const char ALERT_ABBREV[] = "AL";
 
+const char AUTOCOMPLETE_NAME[]   = "AutoComplete";
+const char AUTOCOMPLETE_ABBREV[] = "AC";
+
 const char AUTOINDENT_NAME[]   = "AutoIndent";
 const char AUTOINDENT_ABBREV[] = "AI";
 
@@ -87,6 +90,12 @@ const char DELETEEOL_ABBREV[] = "DE";
 const char DELETELINE_NAME[]   = "DeleteLine";
 const char DELETELINE_ABBREV[] = "DL";
 
+const char DELETENEXTWORD_NAME[]   = "DeleteNextWord";
+const char DELETENEXTWORD_ABBREV[] = "DNW";
+
+const char DELETEPREVWORD_NAME[]   = "DeletePrevWord";
+const char DELETEPREVWORD_ABBREV[] = "DPW";
+
 const char DOUNDO_NAME[]   = "DoUndo";
 const char DOUNDO_ABBREV[] = "DU";
 
@@ -153,6 +162,12 @@ const char INSERTLINE_ABBREV[] = "IL";
 const char INSERTSTRING_NAME[]   = "InsertString";
 const char INSERTSTRING_ABBREV[] = "IS";
 
+const char INSERTTAB_NAME[]   = "InsertTab";
+const char INSERTTAB_ABBREV[] = "IT";
+
+const char KEYCODE_NAME[]   = "KeyCode";
+const char KEYCODE_ABBREV[] = "KC";
+
 const char LINEDOWN_NAME[]   = "LineDown";
 const char LINEDOWN_ABBREV[] = "LD";
 
@@ -306,6 +321,9 @@ const char REPLACEALL_ABBREV[] = "RA";
 const char REPLACEONCE_NAME[]   = "ReplaceOnce";
 const char REPLACEONCE_ABBREV[] = "R1";
 
+const char REQUESTORDER_NAME[]   = "RequestOrder";
+const char REQUESTORDER_ABBREV[] = "RQO";
+
 const char RIGHTMARGIN_NAME[]   = "RightMargin";
 const char RIGHTMARGIN_ABBREV[] = "RM";
 
@@ -351,6 +369,9 @@ const char SYNTAX_ABBREV[] = "SY";
 const char SYSTEM_NAME[]   = "System";
 const char SYSTEM_ABBREV[] = "SYS";
 
+const char TABS_NAME[]   = "Tabs";
+const char TABS_ABBREV[] = "TAB";
+
 const char TABSIZE_NAME[]   = "TabSize";
 const char TABSIZE_ABBREV[] = "TS";
 
@@ -405,41 +426,43 @@ const char WORDWRAP_ABBREV[] = "WW";
 /* These are extras that are very useful in the default menus and key bindings. */
 
 const char PLAYONCE_ABBREV[] = "PL 1";
-const char CENTERVIEW_ABBREV[] = "AV C";
+const char MIDDLEVIEW_ABBREV[] = "AV M";
 
 /* This is the NULL-terminated, ordered list of names, useful for help etc. */
 
 const char * const command_names[ACTION_COUNT+1] = {
-  ABOUT_NAME,  ADJUSTVIEW_NAME,  ALERT_NAME,  AUTOINDENT_NAME,
-  AUTOPREFS_NAME,  BACKSPACE_NAME,  BEEP_NAME,  BINARY_NAME,
-  CAPITALIZE_NAME,  CASESEARCH_NAME,  CENTER_NAME,  CLEAR_NAME,
-  CLIPNUMBER_NAME,  CLOSEDOC_NAME,  COPY_NAME,  CRLF_NAME,
-  CUT_NAME,  DELETECHAR_NAME,  DELETEEOL_NAME,  DELETELINE_NAME,
-  DOUNDO_NAME,  ERASE_NAME,  ESCAPE_NAME,  ESCAPETIME_NAME,
-  EXEC_NAME,  EXIT_NAME,  FASTGUI_NAME,  FIND_NAME,
-  FINDREGEXP_NAME,  FLAGS_NAME,  FLASH_NAME,  FREEFORM_NAME,
-  GOTOBOOKMARK_NAME,  GOTOCOLUMN_NAME,  GOTOLINE_NAME,  GOTOMARK_NAME,
-  HELP_NAME,  HEXCODE_NAME,  INSERT_NAME,  INSERTCHAR_NAME,
-  INSERTLINE_NAME,  INSERTSTRING_NAME,  LINEDOWN_NAME,  LINEUP_NAME,
-  LOADAUTOPREFS_NAME,  LOADPREFS_NAME,  MACRO_NAME,  MARK_NAME,
-  MARKVERT_NAME,  MATCHBRACKET_NAME,  MODIFIED_NAME,  MOVEBOS_NAME,
-  MOVEEOF_NAME,  MOVEEOL_NAME,  MOVEEOW_NAME,  MOVEINCDOWN_NAME,
-  MOVEINCUP_NAME,  MOVELEFT_NAME,  MOVERIGHT_NAME,  MOVESOF_NAME,
-  MOVESOL_NAME,  MOVETOS_NAME,  NEWDOC_NAME,  NEXTDOC_NAME,
-  NEXTPAGE_NAME,  NEXTWORD_NAME,  NOFILEREQ_NAME,  NOP_NAME,
-  OPEN_NAME,  OPENCLIP_NAME,  OPENMACRO_NAME,  OPENNEW_NAME,
-  PAGEDOWN_NAME,  PAGEUP_NAME,  PARAGRAPH_NAME,  PASTE_NAME,
-  PASTEVERT_NAME,  PLAY_NAME,  POPPREFS_NAME,  PRESERVECR_NAME,
-  PREVDOC_NAME,  PREVPAGE_NAME,  PREVWORD_NAME,  PUSHPREFS_NAME,
-  QUIT_NAME,  READONLY_NAME,  RECORD_NAME,  REDO_NAME,
-  REFRESH_NAME,  REPEATLAST_NAME,  REPLACE_NAME,  REPLACEALL_NAME,
-  REPLACEONCE_NAME,  RIGHTMARGIN_NAME,  SAVE_NAME,  SAVEAS_NAME,
-  SAVEAUTOPREFS_NAME,  SAVECLIP_NAME,  SAVEDEFPREFS_NAME,  SAVEMACRO_NAME,
-  SAVEPREFS_NAME,  SEARCHBACK_NAME,  SELECTDOC_NAME,  SETBOOKMARK_NAME,
-  STATUSBAR_NAME,  SUSPEND_NAME,  SYNTAX_NAME,  SYSTEM_NAME,
-  TABSIZE_NAME,  THROUGH_NAME,  TOGGLESEOF_NAME,  TOGGLESEOL_NAME,
-  TOLOWER_NAME,  TOUPPER_NAME,  TURBO_NAME,  UNDELLINE_NAME,
-  UNDO_NAME,  UNLOADMACROS_NAME,  UNSETBOOKMARK_NAME,  UTF8_NAME,
-  UTF8AUTO_NAME,  UTF8IO_NAME,  VERBOSEMACROS_NAME,  VISUALBELL_NAME,
-  WORDWRAP_NAME,  NULL
+  ABOUT_NAME,  ADJUSTVIEW_NAME,  ALERT_NAME,  AUTOCOMPLETE_NAME,
+  AUTOINDENT_NAME,  AUTOPREFS_NAME,  BACKSPACE_NAME,  BEEP_NAME,
+  BINARY_NAME,  CAPITALIZE_NAME,  CASESEARCH_NAME,  CENTER_NAME,
+  CLEAR_NAME,  CLIPNUMBER_NAME,  CLOSEDOC_NAME,  COPY_NAME,
+  CRLF_NAME,  CUT_NAME,  DELETECHAR_NAME,  DELETEEOL_NAME,
+  DELETELINE_NAME,  DELETENEXTWORD_NAME,  DELETEPREVWORD_NAME,  DOUNDO_NAME,
+  ERASE_NAME,  ESCAPE_NAME,  ESCAPETIME_NAME,  EXEC_NAME,
+  EXIT_NAME,  FASTGUI_NAME,  FIND_NAME,  FINDREGEXP_NAME,
+  FLAGS_NAME,  FLASH_NAME,  FREEFORM_NAME,  GOTOBOOKMARK_NAME,
+  GOTOCOLUMN_NAME,  GOTOLINE_NAME,  GOTOMARK_NAME,  HELP_NAME,
+  HEXCODE_NAME,  INSERT_NAME,  INSERTCHAR_NAME,  INSERTLINE_NAME,
+  INSERTSTRING_NAME,  INSERTTAB_NAME,  KEYCODE_NAME,  LINEDOWN_NAME,
+  LINEUP_NAME,  LOADAUTOPREFS_NAME,  LOADPREFS_NAME,  MACRO_NAME,
+  MARK_NAME,  MARKVERT_NAME,  MATCHBRACKET_NAME,  MODIFIED_NAME,
+  MOVEBOS_NAME,  MOVEEOF_NAME,  MOVEEOL_NAME,  MOVEEOW_NAME,
+  MOVEINCDOWN_NAME,  MOVEINCUP_NAME,  MOVELEFT_NAME,  MOVERIGHT_NAME,
+  MOVESOF_NAME,  MOVESOL_NAME,  MOVETOS_NAME,  NEWDOC_NAME,
+  NEXTDOC_NAME,  NEXTPAGE_NAME,  NEXTWORD_NAME,  NOFILEREQ_NAME,
+  NOP_NAME,  OPEN_NAME,  OPENCLIP_NAME,  OPENMACRO_NAME,
+  OPENNEW_NAME,  PAGEDOWN_NAME,  PAGEUP_NAME,  PARAGRAPH_NAME,
+  PASTE_NAME,  PASTEVERT_NAME,  PLAY_NAME,  POPPREFS_NAME,
+  PRESERVECR_NAME,  PREVDOC_NAME,  PREVPAGE_NAME,  PREVWORD_NAME,
+  PUSHPREFS_NAME,  QUIT_NAME,  READONLY_NAME,  RECORD_NAME,
+  REDO_NAME,  REFRESH_NAME,  REPEATLAST_NAME,  REPLACE_NAME,
+  REPLACEALL_NAME,  REPLACEONCE_NAME,  REQUESTORDER_NAME,  RIGHTMARGIN_NAME,
+  SAVE_NAME,  SAVEAS_NAME,  SAVEAUTOPREFS_NAME,  SAVECLIP_NAME,
+  SAVEDEFPREFS_NAME,  SAVEMACRO_NAME,  SAVEPREFS_NAME,  SEARCHBACK_NAME,
+  SELECTDOC_NAME,  SETBOOKMARK_NAME,  STATUSBAR_NAME,  SUSPEND_NAME,
+  SYNTAX_NAME,  SYSTEM_NAME,  TABS_NAME,  TABSIZE_NAME,
+  THROUGH_NAME,  TOGGLESEOF_NAME,  TOGGLESEOL_NAME,  TOLOWER_NAME,
+  TOUPPER_NAME,  TURBO_NAME,  UNDELLINE_NAME,  UNDO_NAME,
+  UNLOADMACROS_NAME,  UNSETBOOKMARK_NAME,  UTF8_NAME,  UTF8AUTO_NAME,
+  UTF8IO_NAME,  VERBOSEMACROS_NAME,  VISUALBELL_NAME,  WORDWRAP_NAME,
+  NULL
 };
