@@ -1,7 +1,7 @@
 /* Extern's for names and abbreviations of all the commands.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2005 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -30,6 +30,9 @@ extern const char ADJUSTVIEW_ABBREV[];
 extern const char ALERT_NAME[];
 extern const char ALERT_ABBREV[];
 
+extern const char AUTOCOMPLETE_NAME[];
+extern const char AUTOCOMPLETE_ABBREV[];
+
 extern const char AUTOINDENT_NAME[];
 extern const char AUTOINDENT_ABBREV[];
 
@@ -81,6 +84,12 @@ extern const char DELETEEOL_ABBREV[];
 extern const char DELETELINE_NAME[];
 extern const char DELETELINE_ABBREV[];
 
+extern const char DELETENEXTWORD_NAME[];
+extern const char DELETENEXTWORD_ABBREV[];
+
+extern const char DELETEPREVWORD_NAME[];
+extern const char DELETEPREVWORD_ABBREV[];
+
 extern const char DOUNDO_NAME[];
 extern const char DOUNDO_ABBREV[];
 
@@ -147,6 +156,12 @@ extern const char INSERTLINE_ABBREV[];
 extern const char INSERTSTRING_NAME[];
 extern const char INSERTSTRING_ABBREV[];
 
+extern const char INSERTTAB_NAME[];
+extern const char INSERTTAB_ABBREV[];
+
+extern const char KEYCODE_NAME[];
+extern const char KEYCODE_ABBREV[];
+
 extern const char LINEDOWN_NAME[];
 extern const char LINEDOWN_ABBREV[];
 
@@ -300,6 +315,9 @@ extern const char REPLACEALL_ABBREV[];
 extern const char REPLACEONCE_NAME[];
 extern const char REPLACEONCE_ABBREV[];
 
+extern const char REQUESTORDER_NAME[];
+extern const char REQUESTORDER_ABBREV[];
+
 extern const char RIGHTMARGIN_NAME[];
 extern const char RIGHTMARGIN_ABBREV[];
 
@@ -345,6 +363,9 @@ extern const char SYNTAX_ABBREV[];
 extern const char SYSTEM_NAME[];
 extern const char SYSTEM_ABBREV[];
 
+extern const char TABS_NAME[];
+extern const char TABS_ABBREV[];
+
 extern const char TABSIZE_NAME[];
 extern const char TABSIZE_ABBREV[];
 
@@ -399,7 +420,7 @@ extern const char WORDWRAP_ABBREV[];
 
 /* These are extras that are very useful in the default menus and key bindings. */
 extern const char PLAYONCE_ABBREV[];
-extern const char CENTERVIEW_ABBREV[];
+extern const char MIDDLEVIEW_ABBREV[];
 
 /* This is the NULL-terminated, ordered list of names, useful for help etc. */
 
