@@ -1,7 +1,7 @@
 /* Navigation functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -581,7 +581,8 @@ int move_bos(buffer * const b) {
    of the file is visible in the terminal window. */
 
 int adjust_view(buffer * const b, const unsigned char *p) {
-	int i, disp, rc = OK;
+	int i, disp, mag, rc = OK;
+	char *q;
 
 	b->y_wanted = 0;
 
@@ -589,15 +590,16 @@ int adjust_view(buffer * const b, const unsigned char *p) {
 	  
 	while(*p) {
 		disp = 0;
+		mag = max(0,strtol(p+1, &q, 0));
 		switch (*p) {
 			case 't' :
 			case 'T' :
 				/* Shift the view so that the current line is displayed at the top. */
-				disp = -b->cur_y;
+				disp = mag ? -min(mag,b->cur_y) : -b->cur_y;
 				break;
 
-			case 'c' :
-			case 'C' :
+			case 'm' :
+			case 'M' :
 				/* Shift the view so that the current line is displayed at the center. */
 				disp = (ne_lines - 2) / 2 - b->cur_y;
 				break;
@@ -605,20 +607,22 @@ int adjust_view(buffer * const b, const unsigned char *p) {
 			case 'b' :
 			case 'B' :
 				/* Shift the view so that the current line is displayed at the bottom. */
-				disp = (ne_lines - 2) - b->cur_y;
+				disp = mag ? min(mag,(ne_lines -2) - b->cur_y) : (ne_lines - 2) - b->cur_y;
 				break;
 
 			case 'l' :
 			case 'L' :
-				/* Shift the view as far left as possible. */
-				while (b->cur_x >= b->opt.tab_size) {
+				/* Shift the view as far left as possible, or mag columns. */
+				if (mag == 0) mag = b->cur_x;
+				while (b->cur_x >= b->opt.tab_size && mag > 0) {
 					b->win_x += b->opt.tab_size;
 					b->cur_x -= b->opt.tab_size;
+					mag      -= b->opt.tab_size;
 				}
 				break;
 
-			case 'm' :
-			case 'M' :
+			case 'c' :
+			case 'C' :
 			   /* Shift the view as far left as possible. This way we don't have to
 			      deal with figuring out which side of Middle the view started on. */
 				while (b->cur_x >= b->opt.tab_size) {
@@ -636,8 +640,10 @@ int adjust_view(buffer * const b, const unsigned char *p) {
 
 			case 'r' :
 			case 'R' :
-				/* Shift the view as far right as possible. */
-				while (b->cur_x < ne_columns - ne_columns % b->opt.tab_size && b->win_x >= b->opt.tab_size) {
+				/* Shift the view as far right as possible, or mag columns. */
+				if (mag == 0) mag = b->win_x;
+				while (b->cur_x < ne_columns - ne_columns % b->opt.tab_size && b->win_x >= b->opt.tab_size && mag > 0) {
+					mag      -= b->opt.tab_size;
 					b->win_x -= b->opt.tab_size;
 					b->cur_x += b->opt.tab_size;
 				}
@@ -663,7 +669,7 @@ int adjust_view(buffer * const b, const unsigned char *p) {
 				b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
 			}
 		}
-		p++;
+		p = q;
 	}
 
 	if (b == cur_buffer) update_window(b);
@@ -926,7 +932,7 @@ int search_word(buffer * const b, const int dir) {
 	if (pos >= ld->line_len) pos = ld->line_len;
 	else {
 		c = get_char(&ld->line[pos], b->encoding);
-		if (ne_ispunct(c, b->encoding) || ne_isspace(c, b->encoding)) space_skipped = TRUE;
+		if (!ne_isword(c, b->encoding)) space_skipped = TRUE;
 	}
 
 	pos = (dir > 0 ? next_pos : prev_pos)(ld->line, pos, b->encoding);
@@ -936,11 +942,11 @@ int search_word(buffer * const b, const int dir) {
 	while(y < b->num_lines && y >= 0) {
 		while(pos < ld->line_len && pos >= 0) {
 			c = get_char(&ld->line[pos], b->encoding);
-			if (ne_ispunct(c, b->encoding) || ne_isspace(c, b->encoding)) space_skipped = TRUE;
+			if (!ne_isword(c, b->encoding)) space_skipped = TRUE;
 			else word_started = TRUE;
 
 			if (dir > 0) {
-				if (space_skipped && !(ne_ispunct(c, b->encoding) || ne_isspace(c, b->encoding))) {
+				if (space_skipped && ne_isword(c, b->encoding)) {
 					goto_line(b, y);
 					goto_pos(b, pos);
 					return OK;
@@ -948,7 +954,7 @@ int search_word(buffer * const b, const int dir) {
 			}
 			else {
 				if (word_started) {
-					if (ne_ispunct(c, b->encoding) || ne_isspace(c, b->encoding)) {
+					if (!ne_isword(c, b->encoding)) {
 						goto_line(b, y);
 						goto_pos(b, pos + 1);
 						return OK;
@@ -956,7 +962,7 @@ int search_word(buffer * const b, const int dir) {
 					else if (pos == 0) {
 						goto_line(b, y);
 						goto_pos(b, 0);
-						return 0;
+						return OK;
 					}
 				}
 			}
@@ -989,11 +995,11 @@ void move_to_eow(buffer * const b) {
 	line_desc *ld = b->cur_line_desc;
    int pos = b->cur_pos, c;
 
-	if (pos >= ld->line_len || ne_ispunct(c = get_char(&ld->line[pos], b->encoding), b->encoding) || ne_isspace(c, b->encoding)) return;
+	if (pos >= ld->line_len || !ne_isword(c = get_char(&ld->line[pos], b->encoding), b->encoding)) return;
 
 	while(pos < ld->line_len) {
 		c = get_char(&ld->line[pos], b->encoding);
-		if (ne_ispunct(c, b->encoding) || ne_isspace(c, b->encoding)) break;
+		if (!ne_isword(c, b->encoding)) break;
 		pos += b->encoding == ENC_UTF8 ? utf8len(ld->line[pos]) : 1;
 	}
 
