@@ -1,7 +1,7 @@
 /* main(), global initialization and global buffer functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -21,11 +21,12 @@
 	02111-1307, USA.  */
 
 
+#include "regex.h"
+
 #include "ne.h"
 
 #include "version.h"
 #include "termchar.h"
-#include "regex.h"
 
 #include <signal.h>
 #include <limits.h>
@@ -37,7 +38,7 @@ message disappears as soon as any key is typed. */
 
 char *NO_WARRANTY_msg[] = {	PROGRAM_NAME " " VERSION ".",
 										"Copyright (C) 1993-1998 Sebastiano Vigna",
-										"Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna",
+										"Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna",
 										"",
 										"This program is free software; you can redistribute it and/or modify",
 										"it under the terms of the GNU General Public License as published by",
@@ -84,6 +85,15 @@ list buffers = { (node *)&buffers.tail, NULL, (node *)&buffers.head };
 list clips = { (node *)&clips.tail, NULL, (node *)&clips.head };
 list macros = { (node *)&macros.tail, NULL, (node *)&macros.head };
 
+/* global prefs, only saved in ~/.ne/.default#ap if their
+   current setting differs from these defaults. Make sure these
+   defaults match the conditionals in prefs.c:save_prefs(). */
+int req_order;
+int fast_gui;
+int status_bar = TRUE;
+int verbose_macros = TRUE;
+/* end of global prefs */
+
 buffer *cur_buffer;
 int turbo;
 int do_syntax = TRUE;
@@ -230,8 +240,14 @@ int main(int argc, char **argv) {
 		}
 	}
 
+#ifdef NE_TEST
+	/* Dump the builtin menu and key bindings to compare to
+	   doc/default.menus and doc/default.keys. */
+	dump_config();
+#endif
+
 	/* Unless --noconfig was specified, we try to configure the
-	menus and the keyboard. Note that these functions can exit() on error. */
+	   menus and the keyboard. Note that these functions can exit() on error. */
 
 	if (!no_config) {
 		get_menu_configuration(menu_conf_name);
@@ -280,15 +296,11 @@ int main(int argc, char **argv) {
 	clear_entire_screen();
 
 
-#ifndef _AMIGA
-
 	/* This function sets fatal_code() as signal interrupt handler
 	for all the dangerous signals (SIGILL, SIGSEGV etc.). */
 
 	set_fatal_code();
 
-#endif
-
 	load_auto_prefs(cur_buffer, DEF_PREFS_NAME);
 
 	if (argc > 1) {
@@ -306,7 +318,7 @@ int main(int argc, char **argv) {
 				if (!first_file) do_action(cur_buffer, NEWDOC_A, -1, NULL);
 				else first_file = FALSE;
 
-				do_action(cur_buffer, OPEN_A, -1, str_dup(argv[i]));
+				do_action(cur_buffer, OPEN_A, 0, str_dup(argv[i]));
 			}
 		}
 
@@ -333,15 +345,22 @@ int main(int argc, char **argv) {
 		displaying_info = TRUE;
 
 		for(i = 0; NO_WARRANTY_msg[i]; i++) {
+			if (i == ne_lines - 1) break;
 			move_cursor(i, 0);
 			output_string(NO_WARRANTY_msg[i], FALSE);
 		}
-		move_cursor(++i,0);
-		if (exists_gprefs_dir()) {
-			output_string("Global Directory: ", FALSE);
-			output_string(exists_gprefs_dir(), FALSE);
+		if (++i < ne_lines - 1) {
+			move_cursor(i, 0);
+			if (exists_gprefs_dir()) {
+				output_string("Global Directory: ", FALSE);
+				output_string(exists_gprefs_dir(), FALSE);
+			}
+			else {
+				output_string("Global directory \"", FALSE);
+				output_string(get_global_dir(), FALSE);
+				output_string("\" not found!", FALSE);
+			}
 		}
-		else output_string("Global directory not found!", FALSE);
 	}
 
 	while(TRUE) {
@@ -367,6 +386,7 @@ int main(int argc, char **argv) {
 			displaying_info = FALSE;
 
 			for(i = 0; i < sizeof NO_WARRANTY_msg / sizeof *NO_WARRANTY_msg + 2; i++) {
+				if (i == ne_lines - 1) break;
 				move_cursor(i, 0);
 				clear_to_eol();
 			}
@@ -380,9 +400,12 @@ int main(int argc, char **argv) {
 			break;
 			
 		case ALPHA:
-		case TAB:
 			print_error(do_action(cur_buffer, INSERTCHAR_A, c, NULL));
 			break;
+
+		case TAB:
+			print_error(do_action(cur_buffer, INSERTTAB_A, 1, NULL));
+			break;
 			
 		case RETURN:
 			print_error(do_action(cur_buffer, INSERTLINE_A, -1, NULL));
