@@ -1,7 +1,7 @@
 /* Main typedefs and defines.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -51,17 +51,8 @@
 #include "termchar.h"
 #include "utf8.h"
 
-#ifdef _AMIGA
-#define CURDIR ""
-#define _S_IFDIR S_IFDIR
-#define S_ISDIR(s) (s & _S_IFDIR)
-#ifdef strcmp
-#undef strcmp
-#endif
-#else
 #define CURDIR "."
 #include <unistd.h>
-#endif
 
 #ifndef min
 #define   min(a,b)    ((a)<(b)?(a):(b))
@@ -83,6 +74,8 @@ typedef enum {
 	 ALPHA, COMMAND, RETURN, TAB, IGNORE, INVALID, INPUT_CLASS_COUNT
 } input_class;
 
+extern const char *input_class_names[];
+
 /* The name of the default preferences file name. */
 
 #define DEF_PREFS_NAME			".default"
@@ -177,12 +170,15 @@ typedef struct {
 } char_stream;
 
 
+#ifndef NDEBUG
 #define assert_char_stream(cs) {if ((cs)) {\
 	assert((cs)->len<=(cs)->size);\
 	assert((cs)->len >= 0);\
 	assert(((cs)->size == 0) == ((cs)->stream == NULL));\
 }}
-
+#else
+#define assert_char_stream(cs) ;
+#endif
 
 
 /* This structure defines a line descriptor; it is a node containing a pointer
@@ -208,14 +204,16 @@ typedef struct {
 	int line_len;
 } no_syntax_line_desc;
 
+#ifndef NDEBUG
 #define assert_line_desc(ld, encoding) {if ((ld)) {		\
 	assert((ld)->line_len >= 0);\
 	assert(((ld)->line == NULL) == ((ld)->line_len == 0));\
 	assert(((ld)->line_len == 0) || ((ld)->line[0] != 0 && (ld)->line[(ld)->line_len - 1] != 0));\
-	assert(((ld)->line_len == 0) || ((ld)->line[0] != 0 && (ld)->line[(ld)->line_len - 1] != 0)); \
 	if (encoding == ENC_UTF8) { int i = 0; while(i < (ld)->line_len) { assert(utf8len((ld)->line[i]) > 0); i = next_pos((ld)->line, i, encoding);} } \
 }}
-
+#else
+#define assert_line_desc(ld, encoding) ;
+#endif
 
 
 /* This structure defines a pool of line descriptors. pool points to an
@@ -231,13 +229,16 @@ typedef struct {
 	line_desc *pool;
 } line_desc_pool;
 
+#ifndef NDEBUG
 #define assert_line_desc_pool(ldp) {if ((ldp)) {\
 	assert((ldp)->allocated_items <= (ldp)->size);\
 	assert((ldp)->allocated_items == (ldp)->size || (ldp)->free_list.head->next);\
 	assert((ldp)->pool != NULL);\
 	assert((ldp)->size != 0);\
 }}
-
+#else
+#define assert_line_desc_pool(ldp) ;
+#endif
 
 
 
@@ -254,6 +255,7 @@ typedef struct {
 	unsigned char *pool;
 } char_pool;
 
+#ifndef NDEBUG
 #define assert_char_pool(cp) {if ((cp)) {\
 	assert((cp)->first_used<=(cp)->first_used);\
 	assert((cp)->pool[(cp)->first_used] != 0);\
@@ -265,7 +267,9 @@ typedef struct {
 	assert((cp)->pool != NULL);\
 	assert((cp)->size != 0);\
 }}
-
+#else
+#define assert_char_pool(cp) ;
+#endif
 
 
 /* This structure defines a macro. A macro is just a stream plus a node, a
@@ -278,8 +282,11 @@ typedef struct struct_macro_desc {
 	char_stream *cs;
 } macro_desc;
 
+#ifndef NDEBUG
 #define assert_macro_desc(md) if (md) assert_char_stream((md)->cs);
-
+#else
+#define assert_macro_desc(md) ;
+#endif
 
 
 /* This structure defines a clip. Clip are numbered from 0 onwards, and
@@ -291,11 +298,14 @@ typedef struct {
 	char_stream *cs;
 } clip_desc;
 
+#ifndef NDEBUG
 #define assert_clip_desc(cd) {if ((cd)) {\
 	assert((cd)->n >= 0);\
 	assert_char_stream((cd)->cs);\
 }}
-
+#else
+#define assert_clip_desc(cd) ;
+#endif
 
 /* An undo step is given by a position, a transformation which can be
 INSERT_CHAR or DELETE_CHAR and the length of the stream to which the
@@ -337,6 +347,7 @@ typedef struct {
 	int last_save_step;
 } undo_buffer;
 
+#ifndef NDEBUG
 #define assert_undo_buffer(ub) {if ((ub)) {\
 	assert((ub)->cur_step<=(ub)->last_step);\
 	assert((ub)->cur_stream<=(ub)->last_stream);\
@@ -346,7 +357,9 @@ typedef struct {
 	assert((ub)->last_stream<=(ub)->streams_size);\
 	assert_char_stream(&(ub)->redo);\
 }}
-
+#else
+#define assert_undo_buffer(ub) ;
+#endif
 
 /* This structure defines all the per document options which can be
 used with PushPrefs and PopPrefs. */
@@ -357,29 +370,31 @@ typedef struct {
 	int right_margin;
 	unsigned int
 		free_form:1,       /* Editing is free form (cursor can be anywhere) */
-		status_bar:1,      /* The status bar is displayed */
 		hex_code:1,        /* Show hexadecimal code under the cursor */
-		fast_gui:1,        /* GUI should be as fast as possible (phone lines, etc.) */
 		word_wrap:1,       /* Word wrap is on */
 		auto_indent:1,     /* Replicate indentation when creating a new line */
 		preserve_cr:1,     /* Preserve Carriage Returns, don't treat as line terminators. */
 		insert:1,          /* Insert mode */
-		verbose_macros:1,  /* Macro learned are verbose */
 		do_undo:1,         /* Record each action and allow undoing it */
 		auto_prefs:1,      /* Use autoprefs */
 		no_file_req:1,     /* Do not display the file requester */
 		read_only:1,       /* Read-only mode */
 		search_back:1,     /* Last search was backwards */
 		case_search:1,     /* Look at case matching in searches */
+		tabs:1,            /* TAB inserts TABs vs. spaces */
 		binary:1,          /* Load and save in binary mode */
 		utf8auto:1,        /* Try to detect automatically UTF-8 */
 		visual_bell:1;     /* Prefer visible bell to audible */
   } options_t;
 
+#ifndef NDEBUG
 #define assert_options(o) {if ((o)) {\
 	assert((o)->tab_size > 0);\
 	assert_undo_buffer(&(b)->undo);\
 }}
+#else
+#define assert_options(o) ;
+#endif
 
 /* This structure defines a buffer node; a buffer is composed by two lists,
 the list of line descriptor pools and the list of character pools, plus some
@@ -452,6 +467,7 @@ typedef struct {
 } buffer;
 
 
+#ifndef NDEBUG
 #define assert_buffer(b) {if ((b)) {\
 	assert((b)->line_desc_list.head->next == NULL || (b)->cur_line_desc != NULL);\
 	assert((b)->line_desc_list.head->next == NULL || (b)->top_line_desc != NULL);\
@@ -478,6 +494,10 @@ typedef struct {
 	}\
 	assert(b->attr_len < 0 || b->attr_len == calc_char_len(b->cur_line_desc, b->encoding));\
 }}
+#else
+#define assert_buffer(b) ;
+#define assert_buffer_content(b);
+#endif
 
 #include "syntax.h"
 
@@ -493,38 +513,71 @@ extern list buffers, clips, macros;
 
 extern int turbo;
 
+
 /* If true, the current line has changed and care must be taken to update the initial state of the following lines. */
 
 extern int need_attr_update;
 
+
 /* If true, we want the hardwired ANSI terminal, not a real one. */
 
 extern int ansi;
 
+
+/* If true, we want requests by column, otherwise by row. */
+
+extern int req_order;
+
+
+/* The status bar is displayed */
+
+extern int status_bar;
+
+
+/* If true, we want abbreviated screen updates. */
+
+extern int fast_gui;
+
+
+/* Recorded macros use long command names */
+
+extern int verbose_macros;
+
+
 /* If true, we want syntax highlighting. */
 
 extern int do_syntax;
 
+
 /* This flag can be set anywhere to FALSE, and will become TRUE if the user
 hits the interrupt key (usually CTRL-'\'). It is handled through SIGQUIT and
 SIGINT. */
 
 extern unsigned int stop;
 
+
 /* This is set by the signal handler whenever a SIGWINCH happens. */
 
 extern unsigned int window_changed_size;
 
+
 /* This vector associates to an extended key code (as returned by
 get_key_code()) its input class. */
 
 extern const input_class char_class[];
 
+/* A boolean recording whether the last replace was for an empty string 
+	(of course, this can happen only with regular expressions). */
+
+extern int last_replace_empty_match;
+
+
 /* This number defines the macro hash table size. This table can have
 conflicts. */
 
 #define MACRO_HASH_TABLE_SIZE (101)
 
+
 /* Upcasing vectors for the regex library. */
 extern unsigned char localised_up_case[];
 extern const unsigned char ascii_up_case[];
@@ -536,14 +589,6 @@ extern const unsigned char ascii_up_case[];
 #include "utf8.h"
 #include "debug.h"
 
-#ifdef _AMIGA
-#define set_fatal_code()
-#define block_signals()
-#define release_signals()
-#define set_stop()
-#define tilde_expand(x)	(x)
-#endif
-
 /* In the unfortunate case we are compiling in some known system
 	with a completely different handling of binary and ASCII files,
 	we force the use of binary files. */
