@@ -1,7 +1,7 @@
 /* Preferences functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -26,13 +26,7 @@
 
 /* These are the names of ne's autoprefs directory. */
 
-#ifdef _AMIGA
-#include <dos.h>
-#define PREFS_DIR "PROGDIR:.ne/"
-#else
 #define PREFS_DIR ".ne"
-#endif
-
 
 /* This string is appended to the filename extension. It tries to
 be enough strange to avoid clashes with macros. */
@@ -48,7 +42,11 @@ bigger just causes a reallocation. */
 
 #define PREF_FILE_SIZE_GUESS 256
 
+/* If we're saving default prefs, we include global prefs
+	that are not buffer specific. Likewise, if we're saving
+	auto prefs, we don't want to include global prefs. */
 
+static int saving_global;
 
 /* Returns a pointer to the extension of a filename, or NULL if there is no
    extension. Note that filename has to be non NULL. */
@@ -76,26 +74,13 @@ const char *extension(const char * const filename) {
 char *exists_prefs_dir(void) {
 
 	static char *prefs_dir;
-#ifndef _AMIGA
 	char *home_dir;
-#endif
 	struct stat s;
 
 	/* If we have been already called, we already computed the name. */
 
 	if (prefs_dir) return prefs_dir;
 
-#ifdef _AMIGA
-	/* The Amiga case is very easy, due to the presence of the PROGDIR: variable.
-	Everything can be handled statically. */
-
-	if (stat(PREFS_DIR, &s)) {
-		if (mkdir(PREFS_DIR)) return NULL;
-	}
-	else if (!(s.st_mode & _S_IFDIR)) return NULL;
-
-	return prefs_dir = PREFS_DIR;
-#else
 	/* In the UN*X case, we first get the home directory. Then
 	we allocate space for the directory name. */
 
@@ -119,7 +104,6 @@ char *exists_prefs_dir(void) {
 		return strcat(prefs_dir, "/");
 	}
 	else return NULL;
-#endif
 }
 
 
@@ -143,17 +127,6 @@ char *exists_gprefs_dir(void) {
 
 	if ((global_dir = get_global_dir()) && (gprefs_dir = malloc(strlen(global_dir) + 3 ))) {
 		strcpy(gprefs_dir, global_dir);
-#ifdef _AMIGA
-		if (stat(gprefs_dir, &s)) {
-			free(gprefs_dir);
-			return gprefs_dir = NULL;
-		}
-		else if (!(s.st_mode & _S_IFDIR)) {
-			free(gprefs_dir);
-			return gprefs_dir = NULL;
-		}
-		return strcat(gprefs_dir,"/");
-#else
 		if (stat(gprefs_dir, &s)) {
 			free(gprefs_dir);
 			return gprefs_dir = NULL;
@@ -163,7 +136,6 @@ char *exists_gprefs_dir(void) {
 			return gprefs_dir = NULL;
 		}
 		return strcat(gprefs_dir, "/");
-#endif
 	}
 	return NULL;
 }
@@ -184,26 +156,37 @@ int save_prefs(buffer * const b, const char * const name) {
 	if (cs = alloc_char_stream(PREF_FILE_SIZE_GUESS)) {
 		/* We create a macro by recording an action for each kind of flag. */
 
-		if (b->syn) record_action(cs, SYNTAX_A, -1, b->syn->name, b->opt.verbose_macros);
+		if (!saving_global && b->syn) record_action(cs, SYNTAX_A, -1, b->syn->name, verbose_macros);
 
-		record_action(cs, FREEFORM_A,      b->opt.free_form,      NULL, b->opt.verbose_macros);
-		record_action(cs, STATUSBAR_A,     b->opt.status_bar,     NULL, b->opt.verbose_macros);
-		record_action(cs, HEXCODE_A,       b->opt.hex_code,       NULL, b->opt.verbose_macros);
-		record_action(cs, FASTGUI_A,       b->opt.fast_gui,       NULL, b->opt.verbose_macros);
-		record_action(cs, WORDWRAP_A,      b->opt.word_wrap,      NULL, b->opt.verbose_macros);
-		record_action(cs, AUTOINDENT_A,    b->opt.auto_indent,    NULL, b->opt.verbose_macros);
-		record_action(cs, INSERT_A,        b->opt.insert,         NULL, b->opt.verbose_macros);
-		record_action(cs, VERBOSEMACROS_A, b->opt.verbose_macros, NULL, b->opt.verbose_macros);
-		record_action(cs, DOUNDO_A,        b->opt.do_undo,        NULL, b->opt.verbose_macros);
-		record_action(cs, AUTOPREFS_A,     b->opt.auto_prefs,     NULL, b->opt.verbose_macros);
-		record_action(cs, NOFILEREQ_A,     b->opt.no_file_req,    NULL, b->opt.verbose_macros);
-		record_action(cs, CASESEARCH_A,    b->opt.case_search,    NULL, b->opt.verbose_macros);
-		record_action(cs, BINARY_A,        b->opt.binary,         NULL, b->opt.verbose_macros);
-		record_action(cs, TABSIZE_A,       b->opt.tab_size,       NULL, b->opt.verbose_macros);
-		record_action(cs, RIGHTMARGIN_A,   b->opt.right_margin,   NULL, b->opt.verbose_macros);
-		record_action(cs, CLIPNUMBER_A,    b->opt.cur_clip,       NULL, b->opt.verbose_macros);
-		record_action(cs, PRESERVECR_A,    b->opt.preserve_cr,    NULL, b->opt.verbose_macros);
-		record_action(cs, VISUALBELL_A,    b->opt.visual_bell,    NULL, b->opt.verbose_macros);
+		record_action(cs, TABSIZE_A,       b->opt.tab_size,       NULL, verbose_macros);
+		record_action(cs, CLIPNUMBER_A,    b->opt.cur_clip,       NULL, verbose_macros);
+		record_action(cs, RIGHTMARGIN_A,   b->opt.right_margin,   NULL, verbose_macros);
+		record_action(cs, FREEFORM_A,      b->opt.free_form,      NULL, verbose_macros);
+		record_action(cs, HEXCODE_A,       b->opt.hex_code,       NULL, verbose_macros);
+		record_action(cs, WORDWRAP_A,      b->opt.word_wrap,      NULL, verbose_macros);
+		record_action(cs, AUTOINDENT_A,    b->opt.auto_indent,    NULL, verbose_macros);
+		record_action(cs, PRESERVECR_A,    b->opt.preserve_cr,    NULL, verbose_macros);
+		record_action(cs, INSERT_A,        b->opt.insert,         NULL, verbose_macros);
+		record_action(cs, DOUNDO_A,        b->opt.do_undo,        NULL, verbose_macros);
+		record_action(cs, AUTOPREFS_A,     b->opt.auto_prefs,     NULL, verbose_macros);
+		record_action(cs, NOFILEREQ_A,     b->opt.no_file_req,    NULL, verbose_macros);
+      /* Skip read_only */
+      /* Skip search_back */
+		record_action(cs, CASESEARCH_A,    b->opt.case_search,    NULL, verbose_macros);
+		record_action(cs, TABS_A,          b->opt.tabs,           NULL, verbose_macros);
+		record_action(cs, BINARY_A,        b->opt.binary,         NULL, verbose_macros);
+		record_action(cs, UTF8AUTO_A,      b->opt.utf8auto,       NULL, verbose_macros);
+		record_action(cs, VISUALBELL_A,    b->opt.visual_bell,    NULL, verbose_macros);
+		
+		if (saving_global) {
+			/* We only save the global flags that differ from their defaults. */
+			/* Make sure these are in sync with the defaults near the top of ne.c. */
+			if (req_order)       record_action(cs, REQUESTORDER_A,  req_order,      NULL, verbose_macros);
+			if (fast_gui)        record_action(cs, FASTGUI_A,       fast_gui,       NULL, verbose_macros);
+			if (!status_bar)     record_action(cs, STATUSBAR_A,     status_bar,     NULL, verbose_macros);
+			if (!verbose_macros) record_action(cs, VERBOSEMACROS_A, verbose_macros, NULL, verbose_macros);
+			saving_global = 0;
+		}
 
 		error = save_stream(cs, name, b->is_CRLF, FALSE);
 
@@ -218,7 +201,7 @@ int save_prefs(buffer * const b, const char * const name) {
 
 
 /* Loads the given preferences file. The file is just executed, but with the
-   exec_oxnly_options flag set. If b or name are NULL, ERROR is returned. */
+   exec_only_options flag set. If b or name are NULL, ERROR is returned. */
 
 int load_prefs(buffer * const b, const char * const name) {
 
@@ -242,17 +225,25 @@ int load_prefs(buffer * const b, const char * const name) {
 	return error;
 }
 
-/* Loads the given syntax. */
+/* Loads the given syntax, taking care to preserve the old 
+   syntax if the new one cannot be loaded. */
 
 int load_syntax_by_name(buffer * const b, const char * const name) {
+	struct high_syntax *syn;
 	assert_buffer(b);
 	assert(name != NULL);
-	b->syn = load_syntax((char *)name);
-	if (!b->syn) b->syn = load_syntax((char *)ext2syntax(name));
-	return b->syn ? OK : NO_SYNTAX_FOR_EXT;
+	syn = load_syntax((char *)name);
+	if (!syn) syn = load_syntax((char *)ext2syntax(name));
+	if (syn) {
+		b->syn = syn;
+		reset_syntax_states(b);
+		return OK;
+	}
+	return NO_SYNTAX_FOR_EXT;
 }
 
 
+
 /* Performs an automatic preferences operation, which can be loading or saving,
    depending on the function pointed to by prefs_func. The extension given by
    ext is used in order to locate the appropriate file. If ext is NULL, the
@@ -311,6 +302,10 @@ int load_auto_prefs(buffer * const b, const char *name) {
 
 
 int save_auto_prefs(buffer * const b, const char *name) {
+	/* In practice, the only time we call save_auto_prefs with a name is
+	   when we save the default prefs. If that changes, so too must this
+	   method of setting this static flag used by save_prefs. */
+	saving_global = name ? 1 : 0;
 	return do_auto_prefs(b, name, save_prefs);
 }
 
