@@ -1,7 +1,7 @@
 /* Function prototypes
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -23,8 +23,8 @@
 /* actions.c */
 int do_action(buffer *b, action a, int c, unsigned char *p);
 
-/* ansi.c */
-void setup_ansi_term(void);
+/* autocomp.c */
+unsigned char *autocomplete(unsigned char *p, char *req_msg, const int ext, int * const error);
 
 /* buffer.c */
 encoding_type detect_buffer_encoding(const buffer *b);
@@ -102,7 +102,7 @@ int highlight_cmp(HIGHLIGHT_STATE *x, HIGHLIGHT_STATE *y);
 void delay_update();
 void output_line_desc(int row, int col, line_desc *ld, int start, int len, int tab_size, int cleared_at_end, int utf8, const int * const attr, const int * const diff, const int diff_size);
 line_desc *update_partial_line(buffer *b, int n, int start_x, int cleared_at_end, const int differential);
-void update_line(buffer *b, int n, const int differential);
+void update_line(buffer *b, int n, const int cleared_at_end, const int differential);
 void update_window_lines(buffer *b, int start_line, int end_line, int doit);
 void update_syntax_and_lines(buffer *b, line_desc *start_ld, line_desc *end_ld);
 void update_window(buffer *b);
@@ -122,7 +122,8 @@ int match_bracket(buffer *b);
 int word_wrap(buffer *b);
 int paragraph(buffer *b);
 int center(buffer *b);
-int auto_indent_line(buffer * const b, const int line, line_desc * const ld);
+int auto_indent_line(buffer * const b, const int line, line_desc * const ld, const int up_to_col);
+int backtab(buffer *b);
 
 /* errors.c */
 
@@ -218,15 +219,15 @@ char *request_string(const char *prompt, const char *default_string, int accept_
 char *request(const char *prompt, const char *default_string, int alpha_allowed, int completion_allowed, int prefer_utf8);
 
 /* request.c */
-int   request_strings(const char * const * const entries, int num_entries, int max_name_len, int mark_dirs);
+int   request_strings(const char * const * const entries, int num_entries, int default_entry, int max_name_len, int mark_char);
 char *request_files(const char *filename, int use_prefix);
 char *request_file(const buffer *b, const char *prompt, const char *default_name);
 int   request_document(void);
 
 /* search.c */
-int  find(buffer *b, const unsigned char *pattern, int dir, int skip_first);
+int  find(buffer *b, const unsigned char *pattern, const skip_first);
 int  replace(buffer *b, int n, const char *string);
-int  find_regexp(buffer *b, const unsigned char *regex, int dir, int skip_first);
+int  find_regexp(buffer *b, const unsigned char *regex, const int skip_first);
 int  replace_regexp(buffer *b, const char *string);
 
 /* signals.c */
@@ -254,6 +255,7 @@ int delete_from_stream(char_stream *cs, int p, int len);
 int insert_in_stream(char_stream *cs, const char *s, int p, int len);
 
 /* support.c */
+int same_str(const char *p, const char *q);
 char *ne_getcwd(const int bufsize);
 const char *get_global_dir(void);
 const char *tilde_expand(const char *filename);
@@ -261,6 +263,7 @@ const char *file_part(const char *pathname);
 char *str_dup(const char *s);
 int strnlen_ne(const char *s, int n);
 int strcmpp(const void *a, const void *b);
+int strdictcmp(const void *a, const void *b);
 int filenamecmpp(const void *a, const void *b);
 void set_interactive_mode(void);
 void unset_interactive_mode(void);
@@ -286,6 +289,7 @@ int get_char(const unsigned char *s, encoding_type encoding);
 int get_char_width(const unsigned char * const s, const encoding_type encoding);
 int ne_ispunct(const int c, const int encoding);
 int ne_isspace(const int c, const int encoding);
+int ne_isword(const int c, const int encoding);
 
 /* term.c */
 int output_width(int c);
@@ -311,7 +315,7 @@ void output_char(int c, unsigned int attr, int utf8);
 void insert_chars(const unsigned char *start, const unsigned int *attr, int raw_len, int utf8);
 void insert_char(int c, const unsigned int attr, int utf8);
 void delete_chars(int n);
-void ins_del_lines(int vpos, int n);
+int ins_del_lines(int vpos, int n);
 int ttysize(void);
 void term_init(void);
 
