@@ -1,7 +1,7 @@
 /* Requester handling.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -49,69 +49,144 @@ reasonable number of retries. */
 
 We rely on a series of auxiliary functions. */
 
-
-static int x, y, page, names_per_line, names_per_page, num_entries, max_name_len, mark_dirs;
+static int x, y, page, max_names_per_line, max_names_per_col, names_per_page, num_entries, max_name_len, mark_char;
 
 static const char * const *entries;
 
+/* ne traditionally has displayed request entries by row, i.e.
+      a   b   c
+      d   e   f
+      g   h
+   rather than by column, i.e.
+      a   d   g
+      b   e   h
+      c   f
+   which, while easier to program, is somewhat harder to read.
+   The request code was also full of tricky expressions like those in
+   the macros below.
+   
+   This version attempts to switch to a by-row request display, and also
+   to consolidate as many of the tricky expressions into one small set of
+   macros.
+   
+   If true, the DISPLAYBYROW define should exibit the behavior of the older
+   versions of ne. If false, the second set of macros are used, and these implement
+   the by-column request display. For now, this may be considered experimental. */
+
+#define BR_NAMES_PER_LINE(p) max_names_per_line
+#define BR_NAMES_PER_COL(p) max_names_per_col
+#define BR_PXY2N(p,x,y)  (((y) + (p) * max_names_per_col) * max_names_per_line + (x))
+#define BR_N2P(n)        ((n) / names_per_page)
+#define BR_N2X(n)        (((n) % names_per_page) % max_names_per_line)
+#define BR_N2Y(n)        (((n) % names_per_page) / max_names_per_line)
+#define BR_DX(p)         1
+#define BR_DY            max_names_per_line
+
+/*
+    This Perl snippet is useful for tweaking the NAMES_PER_LINE and NAMES_PER_COL macros. The point of
+    the complexity on the last page is to use a rectangle in the upper-left part of the window that's
+    roughly proportional to the window itself. (Prior pages use the entire window of course.)
+    Calculating $x first gives slight priority to taller columns rather than wider lines.
+    
+    Translating the Perl "$x =" and "$y =" to the C macros NAMES_PER_LINE and NAMES_PER_COL,
+    respectively, is a matter of substituting the following:
+      $N  ($M - $n)
+      $n  (num_entries % names_per_page)    The number of actual entries on the last page.
+      $X  max_names_per_line
+      $Y  max_names_per_col
+      $M  names_per_page
+      $x  NAMES_PER_LINE(p)
+      
+#!/usr/bin/perl -w
+use strict;
+my ($X,$Y,$M,$N,$x,$y,$n);
+use integer; # use integer math, like C macros do
+($X,$Y) = (5,9);  # change to test different column/row configurations.
+$M = $X * $Y;
+for $n ( 1 .. $M ) {
+    $N = $M - $n;
+    $x = $X - ($X*($N-1)*($N-1)-$M)/($M*$M);
+    $y = ($n+$x-1) / $x;
+    print  "  n (rows,cols) (capacity => n)\n" if $n == 1;
+    printf "%3d: (%2d,%2d)  (%3d >= %3d)? %s\n", $n, $x, $y, $x*$y, $n, ($x*$y >= $n ? "good" : '**BAD**');
+}
+
+*/
+#define LASTPAGE(p)      ((num_entries / names_per_page) > p ? 0 : 1 )
+
+#define BC_NAMES_PER_LINE(p) (LASTPAGE(p) ? (max_names_per_line - (max_names_per_line*((names_per_page - (num_entries % names_per_page))-1)*((names_per_page - (num_entries % names_per_page))-1)-names_per_page)/(names_per_page*names_per_page)) : max_names_per_line )
+#define BC_NAMES_PER_COL(p)  (LASTPAGE(p) ? (((num_entries % names_per_page)+NAMES_PER_LINE(p)-1) / NAMES_PER_LINE(p)) : max_names_per_col)
+#define BC_PXY2N(p,x,y)   ((p) * names_per_page + (x) * NAMES_PER_COL(p) + (y))
+#define BC_N2P(n)         ((n) / names_per_page)
+#define BC_N2X(n)        (((n) % names_per_page) / NAMES_PER_COL(N2P(n))) 
+#define BC_N2Y(n)        (((n) % names_per_page) % NAMES_PER_COL(N2P(n)))
+#define BC_DX(p)         NAMES_PER_COL(p)
+#define BC_DY            1
+
+
+#define NAMES_PER_LINE(p) (req_order ? BC_NAMES_PER_LINE(p) : BR_NAMES_PER_LINE(p))
+#define NAMES_PER_COL(p)  (req_order ? BC_NAMES_PER_COL(p)  : BR_NAMES_PER_COL(p) )
+#define PXY2N(p,x,y)      (req_order ? BC_PXY2N(p,x,y)      : BR_PXY2N(p,x,y)     )
+#define N2P(n)            (req_order ? BC_N2P(n)            : BR_N2P(n)           )
+#define N2X(n)            (req_order ? BC_N2X(n)            : BR_N2X(n)           )
+#define N2Y(n)            (req_order ? BC_N2Y(n)            : BR_N2Y(n)           )
+#define DX(p)             (req_order ? BC_DX(p)             : BR_DX(p)            )
+#define DY                (req_order ? BC_DY                : BR_DY               )
 
 /* This is the printing function used by the requester. It prints the
 strings from the entries array existing in a certain page (a page contains
-(lines-1)*names_per_line items) with max_name_len maximum width. */
+(lines-1)*max_names_per_line items) with max_name_len maximum width. */
 
 static void print_strings() {
 
-	int i,j;
+	int row,col;
 	const char *p;
 
 	set_attr(0);
-
-	for(i = 0; i < ne_lines - 1; i++) {
-		move_cursor(i, 0);
+	for(row = 0; row < max_names_per_col; row++) {
+		move_cursor(row, 0);
 		clear_to_eol();
-
-		for(j = 0; j < names_per_line; j++) {
-			if ((i + page * (ne_lines - 1)) * names_per_line + j < num_entries) {
-				move_cursor(i, j * max_name_len);
-				p = entries[(i + page * (ne_lines - 1)) * names_per_line + j];
-				if (mark_dirs) set_attr(p[strlen(p) - 1] == '/' ? BOLD : 0);
-				output_string(p, io_utf8);
+      if (row < NAMES_PER_COL(page)) {
+			for(col = 0; col < NAMES_PER_LINE(page); col++) {
+				if (PXY2N(page,col,row) < num_entries) {
+					move_cursor(row, col * max_name_len);
+					p = entries[PXY2N(page,col,row)];
+					if (mark_char) set_attr(p[strlen(p) - 1] == mark_char ? BOLD : 0);
+					output_string(p, io_utf8);
+				}
 			}
 		}
 	}
 }
 
+static void normalize(int n) {
+	int p = page;
 
-
-
-
+	if (n < 0 ) n = 0;
+	if (n >= num_entries ) n = num_entries - 1;
+	x = N2X(n);
+	y = N2Y(n);
+	page = N2P(n);
+	if ( p != page ) print_strings();
+}
+	
 static void request_move_to_sol(void) {
 	x = 0;
 }
 
+
 static void request_move_to_eol(void) {
-	if ((page * (ne_lines - 1) + y + 1) * names_per_line < num_entries) x = names_per_line - 1;
-	else x = (num_entries - 1) % names_per_line;
+	while (x < NAMES_PER_LINE(page) - 1 && PXY2N(page,x+1,y) < num_entries) {
+		x++;
+	}
 }
 
 static void request_move_to_sof(void) {
-
-	int i = page;
-
-	x = y = page = 0;
-	if (i != page) print_strings(FALSE);
+	normalize(0);
 }
 
-
 static void request_move_to_eof() {
-
-	int i = page;
-
-	page = (num_entries - 1) / names_per_page;
-	y = ((num_entries - 1) % names_per_page) / names_per_line;
-	x = (num_entries - 1) % names_per_line;
-
-	if (i != page) print_strings();
+   normalize(num_entries - 1);
 }
 
 
@@ -120,93 +195,103 @@ static void request_toggle_seof(void) {
 	else request_move_to_sof();
 }
 
-
-static void request_prev_page(int force) {
-	if (!force && y > 0) y = 0;
-	else if (page) {
-		page--;
-		print_strings();
-	}
+static void request_prev_page(void) {
+	if (page == 0 ) normalize(PXY2N(page,0,0));
+	else normalize(PXY2N(page-1,x,y));
 }
 
 
-static void request_next_page(int force) {
-	if (!force && y < ne_lines - 2) y = ne_lines - 2;
-	else if ((page + 1) * names_per_page < num_entries) {
-		page++;
-		print_strings();
-	}
-	if (page * names_per_page + y*names_per_line + x >= num_entries) request_move_to_eof();
+static void request_next_page(void) {
+	normalize(PXY2N(page+1,x,y));
 }
 
 
 static void request_move_up(void) {
-	if (y > 0) y--;
-	else if (page) {
-		y = ne_lines - 2;
-		request_prev_page(TRUE);
-	}
+	normalize(PXY2N(page,x,y) - DY);
 }
 
-
-
 static void request_move_inc_up(void) {
 	if (x == 0) {
 		if (y == 0) request_move_to_sof();
-		else request_prev_page(FALSE);
+		else request_prev_page();
 	}
 	else request_move_to_sol();
 }
 
-
 static void request_move_down(void) {
-	if (page * names_per_page + (y + 1) * names_per_line + x >= num_entries) request_move_to_eof();
-	else if (y < ne_lines - 2) y++;
-	else if (page < num_entries / names_per_page) {
-		y = 0;
-		request_next_page(TRUE);
-	}
+	normalize(PXY2N(page,x,y) + DY);
 }
 
-
-
 void request_move_inc_down(void) {
-
-	if (x == names_per_line - 1) {
-		if (y == ne_lines - 2) request_move_to_eof();
-		else request_next_page(FALSE);
+	if (x == NAMES_PER_LINE(page) - 1) {
+		if (y == NAMES_PER_COL(page) - 1) request_move_to_eof();
+		else request_next_page();
 	}
 	else request_move_to_eol();
 }
 
 
-/* If mark_dirs is TRUE, we bold names ending with a slash. */
-int request_strings(const char * const * const _entries, const int _num_entries, const int _max_name_len, int _mark_dirs) {
+static void request_move_left(void) {
+	if ( x == 0 && y + page > 0 ) {
+		request_move_up();
+		request_move_to_eol();
+	}
+	else {
+		normalize(PXY2N(page,x,y) - DX(page));
+	}
+}
+
+
+static void request_move_right(void) {
+	if (y < NAMES_PER_COL(page) - 1 && PXY2N(page,0,y+1) < num_entries && 
+	    (x == NAMES_PER_LINE(page) - 1 || PXY2N(page,x+1,y) > num_entries -1)  ) {
+		request_move_to_sol();
+		request_move_down();
+	}
+	else if (y == NAMES_PER_COL(page) - 1 && x == NAMES_PER_LINE(page) - 1 && PXY2N(page+1,0,0) < num_entries) {
+		normalize(PXY2N(page+1,0,0));
+	}
+	else if (PXY2N(page,x,y) + DX(page) < num_entries ) {
+		normalize(PXY2N(page,x,y) + DX(page));
+	}
+}
+			
+
+/* If mark_char is not '\0', we bold names ending with it. */
+int request_strings(const char * const * const _entries, const int _num_entries, int n, const int _max_name_len, int _mark_char) {
 
 	action a;
 	input_class ic;
-	int c, i, n;
+	int c, i, ne_lines0, ne_columns0;
 
 	assert(_num_entries > 0);
 
-	x = y = page = 0;
+	ne_lines0 = ne_columns0 = max_names_per_line = max_names_per_col = x = y = page = 0;
 	entries = _entries;
 	num_entries = _num_entries;
-	max_name_len = _max_name_len;
-	mark_dirs = _mark_dirs;
-
-	if (!(names_per_line = ne_columns / (++max_name_len))) names_per_line = 1;
-	names_per_page = names_per_line * (ne_lines - 1);
-
-	print_strings();
+	max_name_len = _max_name_len + 1;
+	mark_char = _mark_char;
 
 	while(TRUE) {
+		if (ne_lines0 != ne_lines || ne_columns0 != ne_columns) {
+			if (ne_lines0 && ne_columns0 ) n = PXY2N(page,x,y);
+			if (!(max_names_per_line = ne_columns / (max_name_len))) max_names_per_line = 1;
+			max_names_per_col = ne_lines - 1;
+			names_per_page = max_names_per_line * max_names_per_col;
+			ne_lines0 = ne_lines;
+			ne_columns0 = ne_columns;
+			page = N2P(n);
+			x = N2X(n);
+			y = N2Y(n);
+			print_strings();
+			print_message(NULL);
+		}
 
 		move_cursor(y, x * max_name_len);
 
 		do c = get_key_code(); while((ic = CHAR_CLASS(c)) == IGNORE || ic == INVALID);
 
-		n = page * names_per_page + y*names_per_line + x;
+		n = PXY2N(page,x,y);
 
 		switch(ic) {
 			case ALPHA:
@@ -216,16 +301,7 @@ int request_strings(const char * const * const _entries, const int _num_entries,
 
 				for(i = 1; i < num_entries; i++)
 					if (localised_up_case[(unsigned char)entries[(n + i) % num_entries][0]] == c) {
-
-						n = (n + i) % num_entries;
-
-						if (n / names_per_page != page) {
-							page = n / names_per_page;
-							print_strings();
-						}
-
-						y = (n % names_per_page) / names_per_line;
-						x = n % names_per_line;
+						normalize((n + i) % num_entries);
 						break;
 					}
 				break;
@@ -242,23 +318,12 @@ int request_strings(const char * const * const _entries, const int _num_entries,
 				if (c < 0) c = -c - 1;
 				if ((a = parse_command_line(key_binding[c], NULL, NULL, FALSE))>=0) {
 					switch(a) {
-
 					case MOVERIGHT_A:
-						if (page * names_per_page + y*names_per_line + x != num_entries - 1) {
-							if (++x == names_per_line) {
-								x = 0;
-								request_move_down();
-							}
-						}
+						request_move_right();
 						break;
 
 					case MOVELEFT_A:
-						if (x + y+page != 0) {
-							if (--x < 0) {
-								x = names_per_line - 1;
-								request_move_up();
-							}
-						}
+						request_move_left();
 						break;
 
 					case MOVESOL_A:
@@ -271,15 +336,15 @@ int request_strings(const char * const * const _entries, const int _num_entries,
 
 					case TOGGLESEOL_A:
 						if (x != 0) x = 0;
-						else x = names_per_line - 1;
+						else request_move_to_eol();
 						break;
 
 					case LINEUP_A:
-						if (page + y != 0) request_move_up();
+						request_move_up();
 						break;
 
 					case LINEDOWN_A:
-						if (page * (ne_lines - 1) + y < (num_entries + names_per_line - 1) / names_per_line - 1) request_move_down();
+						request_move_down();
 						break;
 
 					case MOVEINCUP_A:
@@ -292,12 +357,12 @@ int request_strings(const char * const * const _entries, const int _num_entries,
 
 					case PAGEUP_A:
 					case PREVPAGE_A:
-						request_prev_page(FALSE);
+						request_prev_page();
 						break;
 
 					case PAGEDOWN_A:
 					case NEXTPAGE_A:
-						request_next_page(FALSE);
+						request_next_page();
 						break;
 
 					case MOVESOF_A:
@@ -368,7 +433,7 @@ char *complete(const char *start_prefix) {
 		closedir(d);
 	}
 	
-   if (cur_prefix) {
+	if (cur_prefix) {
 	   result = malloc(strlen(dir_name) + strlen(cur_prefix) + 2);
 	   strcat(strcat(strcpy(result, dir_name), cur_prefix), unique && is_dir ? "/" : "");
 	}
@@ -425,13 +490,6 @@ char *request_files(const char * const filename, int use_prefix) {
 
 					num_entries = max_name_len = total_len = 0;
 
-#ifdef _AMIGA
-					total_len = 2;
-					num_entries++;
-					strcpy(names, "/");
-					entries[0] = names;
-#endif
-
 					stop = FALSE;
 
 					while(!stop && (de = readdir(d))) {
@@ -468,12 +526,10 @@ char *request_files(const char * const filename, int use_prefix) {
 					if (num_entries) {
 						qsort(entries, num_entries, sizeof(char *), filenamecmpp);
 
-						if ((i = request_strings((const char * const *)entries, num_entries, max_name_len, TRUE)) != ERROR) {
+						if ((i = request_strings((const char * const *)entries, num_entries, 0, max_name_len, '/')) != ERROR) {
 							p = entries[i >= 0 ? i : -i - 2];
 							if (p[strlen(p) - 1] == '/' && i >= 0) {
-#ifndef _AMIGA
 								p[strlen(p) - 1] = 0;
-#endif
 								if (chdir(p)) alert();
 								else use_prefix = FALSE;
 								next_dir = TRUE;
@@ -572,8 +628,7 @@ int request_document(void) {
 					b = (buffer *)b->b_node.next;
 				}
 
-				i = request_strings((const char * const *)entries, num_entries, max_name_len, FALSE);
-
+				i = request_strings((const char * const *)entries, num_entries, 0, max_name_len, '\0');
 				reset_window();
 
 				free(names);
