@@ -1,7 +1,7 @@
 /* Miscellaneous support functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2009 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -24,12 +24,7 @@
 #include "ne.h"
 #include "cm.h"
 #include <signal.h>
-
-#ifdef _AMIGA
-#include <proto/dos.h>
-#else
 #include <pwd.h>
-#endif
 
 /* Some systems do not define _POSIX_VDISABLE. We try to establish a reasonable value. */
 
@@ -37,19 +32,6 @@
 #define _POSIX_VDISABLE 0
 #endif
 
-
-/* This is the assert function. It appears that its number of arguments is
-different on different ANSI C compilers. Better to redefine it. */
-
-#ifndef NODEBUG
-
-void __assert(int x, char *expr, char *file, int line) {
-	if (!x) fprintf(stderr, "%s can't be false at line %d of %s!\n", expr, line, file);
-}
-
-#endif
-
-
 /* Returns a pointer to the global ne directory if the environment variable
 	NE_GLOBAL_DIR is set.  If it isn't set, then GLOBALDIR is returned. */
 
@@ -131,8 +113,6 @@ int is_directory(const char * const name) {
 	locally. Note that this function can return the same pointer which is
 	passed, in case no tilde expansion has to be performed. */
 
-#ifndef _AMIGA
-
 const char *tilde_expand(const char * filename) {
 
 	static char *expanded_filename;
@@ -182,33 +162,17 @@ const char *tilde_expand(const char * filename) {
 	return filename;
 }
 
-#endif
-
 
 
 /* Given a pathname, returns a pointer to the real file name (i.e., the pointer
 	points inside the string passed). */
 
 const char *file_part(const char * const pathname) {
-
 	const char *p;
-
 	if (!pathname) return NULL;
-
-#ifdef _AMIGA
-
-	return (const char *)FilePart((char *)pathname);
-
-#else
-
 	p = pathname + strlen(pathname);
-
 	while(p > pathname && *(p - 1) != '/') p--;
-
 	return p;
-
-#endif
-
 }
 
 
@@ -235,6 +199,13 @@ int strnlen_ne(const char *s, int n) {
 	return s - p;
 }
 
+/* Compares strings for equality, but accepts NULLs. */
+
+int same_str(const char *p, const char *q) {
+	if (p == q) return TRUE;
+	if (p == NULL || q == NULL) return FALSE;
+	return strcmp(p, q) == 0;
+}
 
 /* Computes the length of the maximal common prefix of s and	t. */
 
@@ -265,6 +236,15 @@ int strcmpp(const void *a, const void *b) {
 	return strcmp(*(const char **)a, *(const char **)b);
 }
 
+/* Another comparison for qsort, this one does dictionary order. */
+
+int strdictcmp(const void *a, const void *b)	{
+	int ci;
+	
+	if ( ci = strcasecmp(*(char * const *)a, *(char * const *)b) ) return ci;
+	return strcmp(*(char * const *) a, * (char * const *) b);
+}
+                                                                       
 
 /* A filename comparison function for qsort(). It makes "../" the first string, "./" the second string
    and then orders lexicographically. */
@@ -275,43 +255,21 @@ int filenamecmpp(const void *a, const void *b) {
 	if (strcmp(t, "../")==0) return 1;
 	if (strcmp(s, "./")==0) return strcmp(t, "./") == 0 ? 0 : -1;
 	if (strcmp(t, "./")==0) return 1;
-	return strcmp(s, t);
+	/* return strcmp(s, t); */
+	return strdictcmp(a, b);
 }
 
 
 
-#ifdef _AMIGA
-
-/* Supplies a getenv() simulation on the Amiga. Unfortunately, the SAS/C
-	version of getenv() does not look at the local variables, which are the only
-	ones we are really interested in... */
-
-char *getenv(const char * const name) {
-
-	static char buffer[2048];
-
-	if (GetVar((char *)name, buffer, sizeof buffer, 0) > -1) return buffer;
-	else return NULL;
-}
-
-#endif
-
-
-
 /* Sets the "interactive I/O mode" of the terminal. It suitably sets the mode
 	bits of the termios structure, and then transmits various capability strings
 	by calling set_terminal_modes(). This function assumes that the terminfo
 	database has been properly initialized. The old_termios structure records
 	the original state of the terminal interface. */
 
-#ifndef _AMIGA
 static struct termios termios, old_termios;
-#endif
 
 void set_interactive_mode(void) {
-
-#ifndef _AMIGA
-
 	tcgetattr(0, &termios);
 
 	old_termios = termios;
@@ -360,15 +318,9 @@ void set_interactive_mode(void) {
 	signal(SIGTSTP, SIG_IGN);
 
 #ifdef SIGWINCH
+	siginterrupt(SIGWINCH, 1);
 	signal(SIGWINCH, handle_winch);
 #endif
-
-#else
-
-	SetMode(Input(), 1);
-
-#endif
-
 	/* This ensures that a physical read will be performed at each getchar(). */
 
 	setbuf(stdin, NULL);
@@ -406,11 +358,7 @@ void unset_interactive_mode(void) {
 	/* Now we restore all the flags in the termios structure to the state they
 		were before us. */
 
-#ifndef _AMIGA
 	tcsetattr(0, TCSADRAIN, &old_termios);
-#else
-	SetMode(Input(), 0);
-#endif
 
 #ifdef SIGWINCH
 	signal(SIGWINCH, SIG_IGN);
@@ -600,3 +548,13 @@ int ne_isspace(const int c, const int encoding) {
 	return encoding != ENC_UTF8 ? isspace(c) : iswspace(c);
 #endif
 }
+
+/* Returns whether the given character is a "word" character.
+	Word characters are '_' plus any non-punctuation or space.
+   
+	TODO: implement a way for users to specify their own word characters.
+	For now, hardcode '_'.  */
+
+int ne_isword(const int c, const int encoding) {
+	return c == '_' || !(ne_isspace(c, encoding) || ne_ispunct(c, encoding));
+}
