@@ -34,11 +34,10 @@
 #include "info2cap.h"
 #endif
 
-#ifndef _AMIGA
 #include <termios.h>
 #include <sys/ioctl.h>
-#endif
 
+#include "ansi.h"
 #include "termchar.h"
 #include "cm.h"
 #include "utf8.h"
@@ -613,7 +612,6 @@ void clear_end_of_line(const int first_unused_hpos) {
 	int i;
 
 	if (curX >= first_unused_hpos) return;
-	standout_if_wanted();
 
 	if (ne_clr_eol) OUTPUT1 (ne_clr_eol);
 	else {
@@ -874,6 +872,7 @@ void delete_chars (int n) {
 
 	if (n == 0) return;
 
+	standout_if_wanted();
 	if (delete_in_insert_mode) turn_on_insert();
 	else {
 		turn_off_insert();
@@ -884,7 +883,7 @@ void delete_chars (int n) {
 		buf = tparm(ne_parm_dch, n);
 		OUTPUT1(buf);
 	}
-	else while(--n >= 0) OUTPUT1(ne_delete_character);
+	else while(n-- != 0) OUTPUT1(ne_delete_character);
 
 	if (!delete_in_insert_mode) OUTPUT_IF(ne_exit_delete_mode);
 }
@@ -895,30 +894,29 @@ for n lines, given a parametrized and/or a one-line capability
 for that purpose. */
 
 static void do_multi_ins_del(char * const multi, const char * const single, int n) {
-
 	if (multi) {
 		char *const buf = tparm(multi, n);
 		OUTPUT(buf);
 	}
-	else while(--n >= 0) OUTPUT(single);
+	else while(n-- != 0) OUTPUT(single);
 }
 
 
 /* Inserts n lines at vertical position vpos. If n is negative, it deletes -n
 	lines. specified_window is taken into account. This function assumes
-	line_ins_del_ok == TRUE. */
+	line_ins_del_ok == TRUE. Returns TRUE if an insertion/deletion actually happened. */
 
-void ins_del_lines (const int vpos, const int n) {
+int ins_del_lines (const int vpos, const int n) {
 
 	int i = n > 0 ? n : -n;
 
 	assert(line_ins_del_ok);
 	assert(i != 0);
 	assert(vpos < specified_window);
-
-	if (scroll_region_ok && vpos + i >= specified_window) return;
-
-	if (!ne_memory_below && vpos + i >= ne_lines) return;
+	
+	if (scroll_region_ok && vpos + i >= specified_window) return FALSE;
+	
+	if (!ne_memory_below && vpos + i >= ne_lines) return FALSE;
 
 	standout_if_wanted();
 
@@ -927,11 +925,11 @@ void ins_del_lines (const int vpos, const int n) {
 
 		if (n < 0) {
 			move_cursor(specified_window - 1, 0);
-			while (--i >= 0) OUTPUTL(ne_scroll_forward, specified_window - vpos + 1);
+			while (i-- != 0) OUTPUTL(ne_scroll_forward, specified_window - vpos + 1);
 		}
 		else {
 			move_cursor(vpos, 0);
-			while (--i >= 0) OUTPUTL(ne_scroll_reverse, specified_window - vpos + 1);
+			while (i-- != 0) OUTPUTL(ne_scroll_reverse, specified_window - vpos + 1);
 		}
 
 		if (specified_window != ne_lines) set_scroll_region(0, ne_lines - 1);
@@ -960,6 +958,8 @@ void ins_del_lines (const int vpos, const int n) {
 			}
 		}
 	}
+	
+	return TRUE;
 }
 
 
@@ -989,30 +989,42 @@ static void calculate_costs (void) {
 
 
 
-/* Gets the window size using TIOCGWINSZ, or does nothing if TIOCGWINSZ is not
-	available. It is called by the signal handler for SIGWINCH on systems that
-	support it. Return 1 if the window size has changed. */
+/* Gets the window size using TIOCGSIZE, TIOCGWINSZ, or LINES/COLUMNS as a
+	last resort. It is called by the signal handler for SIGWINCH on systems
+	that support it. Return 1 if the window size has changed. */
 
 int ttysize(void) {
-#ifdef TIOCGWINSZ
-	struct winsize size;
-	
-	/* try using the TIOCGWINSZ call, if defined */
+	int l, c;	
+#ifdef TIOCGSIZE
+	/* try using the TIOCGSIZE call, if defined */
+	struct ttysize size;
 	D(fprintf(stderr,"ttysize: CHECKING...\n");)
-	
-	if (ioctl(0, TIOCGWINSZ, &size) >= 0) {
-		D(fprintf(stderr,"ttysize:...size is (%d,%d)\n",size.ws_row,size.ws_col);)
-		if ((ne_lines != size.ws_row) || (ne_columns != size.ws_col)) {
-			ScreenRows = ne_lines    = size.ws_row;
-			ScreenCols = ne_columns  = size.ws_col;
-			set_terminal_window(ne_lines - 1);
-			if (scroll_region_ok) set_scroll_region(0, ne_lines - 1);
-			D(fprintf(stderr,"ttysize: size changed.\n");)
-			return 1;
-		}
-	}
+	if (ioctl(0, TIOCGSIZE, &size)) return 0;
+	l = size.ts_lines;
+	c = size.ts_cols;
+#elif defined(TIOCGWINSZ)
+	/* try using the TIOCGWINSZ call, if defined */
+	struct winsize size;
+	D(fprintf(stderr,"ttysize: CHECKING...\n");)
+	if (ioctl(0, TIOCGWINSZ, &size)) return 0;
+	l = size.ws_row;
+	c = size.ws_col;
+#else
+	/* As a last resort, we try to read LINES and COLUMNS, falling back to the terminal-specified size. */
+	if (! getenv("LINES") || ! getenv("COLUMNS")) return 0;
+	l = strtol(getenv("LINES"), NULL, 10);
+	c = strtol(getenv("COLUMNS"), NULL, 10);
 #endif
-	/* no special way to detect screen size */
+	D(fprintf(stderr,"ttysize:...size is (%d,%d)\n", l, c);)
+	if (((ne_lines != l) || (ne_columns != c)) && l > 0 && c > 0) {
+		ScreenRows = ne_lines    = l;
+		ScreenCols = ne_columns  = c;
+		set_terminal_window(ne_lines - 1);
+		if (scroll_region_ok) set_scroll_region(0, ne_lines - 1);
+		D(fprintf(stderr,"ttysize: size changed.\n");)
+		return 1;
+	}
+	
 	return 0;
 }
 
@@ -1175,8 +1187,6 @@ void term_init (void) {
 	ScreenRows = ne_lines;
 	ScreenCols = ne_columns;
 
-	ttysize();
-
 	if (!ne_bell) ne_bell = "\07";
 
 	if (!ne_scroll_forward) ne_scroll_forward = Down;
